@@ -1,6 +1,6 @@
 # -*- ruby -*-
 #--
-# Copyright 2006 by Chad Fowler, Rich Kilmer, Jim Weirich and others.
+# Copyright 2006, 2008 by Chad Fowler, Rich Kilmer, Jim Weirich and others.
 # All rights reserved.
 # See LICENSE.txt for permissions.
 #++
@@ -718,7 +718,7 @@ module Gem
     @gem_path.uniq!
     @gem_path.each do |path|
       if 0 == File.expand_path(path).index(Gem.user_home)
-        unless win_platform? then
+        if defined?(Etc) && !Etc.getpwuid.nil? then
           # only create by matching user
           next if Etc.getpwuid.uid != File::Stat.new(Gem.user_home).uid
         end
