@@ -3,38 +3,17 @@ require 'rbconfig'
 module JRuby
   class Commands
     class << self
+      dollar_zero = Config::CONFIG['bindir'] + "/#{$0}"
       # Provide method aliases to scripts commonly found in ${jruby.home}/bin.
-      if File.directory?(Config::CONFIG['bindir'])
-        Dir[Config::CONFIG['bindir'] + '/*'].each do |f|
-          if File.file?(f) && File.open(f) {|io| (io.readline rescue "") =~ /^#!.*ruby/}
-            meth = File.basename(f)
-            define_method meth do
-              require 'jruby/extract'
-              JRuby::Extract.new.extract
-              load f
-            end
-          end
-        end
-      else
-        # allow use of 'gem' and 'jirb' without prior extraction
-        def gem
-          require 'jruby/extract'
-          JRuby::Extract.new.extract
-          require 'rubygems'
-          require 'rubygems/gem_runner'
-          Gem.manage_gems
-          Gem::GemRunner.new.run(ARGV)
-        end
-
-        def jirb
-          require 'irb'
-          IRB.start(__FILE__)
+      ruby_bin = File.open(dollar_zero) {|io| (io.readline rescue "") =~ /^#!.*ruby/} rescue nil
+      if ruby_bin
+        define_method File.basename(dollar_zero) do
+          $0 = dollar_zero
+          load dollar_zero
         end
       end
 
       def maybe_install_gems
-        require 'jruby/extract'
-        JRuby::Extract.new.extract
         require 'rubygems'
         ARGV.delete_if do |g|
           begin
@@ -50,8 +29,9 @@ module JRuby
         end
         unless ARGV.reject{|a| a =~ /^-/}.empty?
           ARGV.unshift "install"
-          self.gem
-        end 
+          load Config::CONFIG['bindir'] + "/gem"
+        end
+        generate_bat_stubs
       end
 
       def generate_bat_stubs
@@ -72,8 +52,7 @@ module JRuby
         require 'jruby/extract'
         JRuby::Extract.new(ARGV.first).extract
       end
-
-      alias_method :irb, :jirb
     end
   end
 end
+
