@@ -42,7 +42,6 @@ import org.jruby.anno.JRubyMethod;
 import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.CallbackFactory;
-import org.jruby.runtime.MethodIndex;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
@@ -971,7 +970,7 @@ public class RubyBigDecimal extends RubyNumeric {
         int pow = len / 4;
         array[1] = runtime.newFixnum((pow + 1) * 4);
 
-        return RubyArray.newArray(runtime, array);
+        return RubyArray.newArrayNoCopy(runtime, array);
     }
 
     @JRubyMethod(name = "round", optional = 2)
@@ -1067,7 +1066,7 @@ public class RubyBigDecimal extends RubyNumeric {
         // base
         array[2] = runtime.newFixnum(10);
 
-        return RubyArray.newArray(runtime, array);
+        return RubyArray.newArrayNoCopy(runtime, array);
     }
 
     // it doesn't handle special cases
