@@ -27,10 +27,9 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby;
 
-import org.jruby.exceptions.JumpException;
+import org.jruby.anno.JRubyMethod;
+import org.jruby.anno.JRubyClass;
 import org.jruby.runtime.Block;
-import org.jruby.runtime.CallbackFactory;
-import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 
 /**
@@ -38,21 +37,16 @@ import org.jruby.runtime.builtin.IRubyObject;
  *
  * @author <a href="mailto:ola.bini@ki.se">Ola Bini</a>
  */
+@JRubyClass(name="Continuation")
 public class RubyContinuation {
     public static void createContinuation(Ruby runtime) {
-        RubyModule cContinuation = runtime.defineClass("Continuation",runtime.getObject(),runtime.getObject().getAllocator());
-        CallbackFactory cf = runtime.callbackFactory(RubyContinuation.class);
-        cContinuation.defineMethod("call", cf.getOptSingletonMethod("call"));
-        cContinuation.defineAlias("[]","call");
+        RubyClass cContinuation = runtime.defineClass("Continuation",runtime.getObject(),runtime.getObject().getAllocator());
+        cContinuation.defineAnnotatedMethods(RubyContinuation.class);
+        runtime.setContinuation(cContinuation);
     }
 
+    @JRubyMethod(name = {"call", "[]"}, rest = true, frame = true)
     public static IRubyObject call(IRubyObject recv, IRubyObject[] args, Block unusedBlock) {
-        recv.getRuntime().getWarnings().warn("Continuation.call: Continuations are not implemented in JRuby and will not work");
-        ThreadContext context = recv.getRuntime().getCurrentContext();
-        if(args.length == 1) {
-            throw context.prepareJumpException(JumpException.JumpType.BreakJump, recv.dataGetStruct(), args[0]);
-        } else {
-            throw context.prepareJumpException(JumpException.JumpType.BreakJump, recv.dataGetStruct(), recv.getRuntime().newArray(args));
-        }
+        throw recv.getRuntime().newNotImplementedError("Continuations are not implemented in JRuby and will not work");
     }
 }// RubyContinuation
