@@ -56,7 +56,6 @@ import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ClassIndex;
-import org.jruby.runtime.MethodIndex;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
@@ -143,7 +142,7 @@ public class RubyHash extends RubyObject implements Map {
 
         if (args.length == 1) {
             IRubyObject tmp = TypeConverter.convertToTypeWithCheck(
-                    args[0], runtime.getHash(), MethodIndex.TO_HASH, "to_hash");
+                    args[0], runtime.getHash(), "to_hash");
 
             if (!tmp.isNil()) {
                 RubyHash otherHash = (RubyHash) tmp;
@@ -161,6 +160,11 @@ public class RubyHash extends RubyObject implements Map {
         return hash;
     }
 
+    @JRubyMethod(name = "try_convert", meta = true, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject try_convert(ThreadContext context, IRubyObject recv, IRubyObject args) {
+        return TypeConverter.convertToTypeWithCheck(args, context.getRuntime().getHash(), "to_hash");
+    }
+
     /** rb_hash_new
      *
      */
@@ -871,7 +875,7 @@ public class RubyHash extends RubyObject implements Map {
     @JRubyMethod(name = "[]", required = 1)
     public IRubyObject op_aref(ThreadContext context, IRubyObject key) {
         IRubyObject value;
-        return ((value = internalGet(key)) == null) ? callMethod(context, MethodIndex.DEFAULT, "default", key) : value;
+        return ((value = internalGet(key)) == null) ? callMethod(context, "default", key) : value;
     }
 
     /** rb_hash_hash
@@ -923,14 +927,37 @@ public class RubyHash extends RubyObject implements Map {
         return internalGetEntry(key) == NO_ENTRY ? getRuntime().getFalse() : getRuntime().getTrue();
     }
 
-    private class Found extends RuntimeException {}
+    private static class Found extends RuntimeException {
+        @Override
+        public synchronized Throwable fillInStackTrace() {
+            return null;
+        }
+    }
+
+    private static final Found FOUND = new Found();
+
+    private static class FoundKey extends Found {
+        public final IRubyObject key;
+        FoundKey(IRubyObject key) {
+            super();
+            this.key = key;
+        }
+    }
+
+    private static class FoundPair extends FoundKey {
+        public final IRubyObject value;
+        FoundPair(IRubyObject key, IRubyObject value) {
+            super(key);
+            this.value = value;
+        }
+    }
 
     private boolean hasValue(final ThreadContext context, final IRubyObject expected) {
         try {
             visitAll(new Visitor() {
                 public void visit(IRubyObject key, IRubyObject value) {
                     if (equalInternal(context, value, expected)) {
-                        throw new Found();
+                        throw FOUND;
                     }
                 }
             });
@@ -1038,25 +1065,25 @@ public class RubyHash extends RubyObject implements Map {
         return to_a().sort_bang(block);
     }
 
-    private static class FoundKey extends RuntimeException {
-        public IRubyObject key;
-        FoundKey(IRubyObject key) {
-            super();
-            this.key = key;
-        }
-    }
-
     /** rb_hash_index
      *
      */
-    @JRubyMethod(name = "index", required = 1)
+    @JRubyMethod(name = "index", compat = CompatVersion.RUBY1_8)
     public IRubyObject index(ThreadContext context, IRubyObject expected) {
         IRubyObject key = internalIndex(context, expected);
-        if (key != null) {
-            return key;
-        } else {
-            return getRuntime().getNil();
-        }
+        return key != null ? key : context.getRuntime().getNil();
+    }
+
+    @JRubyMethod(name = "index", compat = CompatVersion.RUBY1_9)
+    public IRubyObject index19(ThreadContext context, IRubyObject expected) {
+        context.getRuntime().getWarnings().warn(ID.DEPRECATED_METHOD, "Hash#index is deprecated; use Hash#key");
+        return key(context, expected);
+    }
+
+    @JRubyMethod(name = "key", compat = CompatVersion.RUBY1_9)
+    public IRubyObject key(ThreadContext context, IRubyObject expected) {
+        IRubyObject key = internalIndex(context, expected);
+        return key != null ? key : context.getRuntime().getNil();
     }
 
     private IRubyObject internalIndex(final ThreadContext context, final IRubyObject expected) {
@@ -1392,6 +1419,52 @@ public class RubyHash extends RubyObject implements Map {
         return result;
     }
 
+    @JRubyMethod(name = "assoc", compat = CompatVersion.RUBY1_9)
+    public IRubyObject assoc(final ThreadContext context, final IRubyObject obj) {
+        try {
+            visitAll(new Visitor() {
+                public void visit(IRubyObject key, IRubyObject value) {
+                    if (equalInternal(context, obj, key)) {
+                        throw new FoundPair(key, value);
+                    }
+                }
+            });
+            return context.getRuntime().getNil();
+        } catch (FoundPair found) {
+            return context.getRuntime().newArray(found.key, found.value);
+        }
+    }
+
+    @JRubyMethod(name = "rassoc", compat = CompatVersion.RUBY1_9)
+    public IRubyObject rassoc(final ThreadContext context, final IRubyObject obj) {
+        try {
+            visitAll(new Visitor() {
+                public void visit(IRubyObject key, IRubyObject value) {
+                    if (equalInternal(context, obj, value)) {
+                        throw new FoundPair(key, value);
+                    }
+                }
+            });
+            return context.getRuntime().getNil();
+        } catch (FoundPair found) {
+            return context.getRuntime().newArray(found.key, found.value);
+        }
+    }
+
+    @JRubyMethod(name = "flatten", compat = CompatVersion.RUBY1_9)
+    public IRubyObject flatten(ThreadContext context) {
+        RubyArray ary = to_a(); 
+        ary.callMethod(context, "flatten!", RubyFixnum.one(context.getRuntime()));
+        return ary;
+    }
+
+    @JRubyMethod(name = "flatten", compat = CompatVersion.RUBY1_9)
+    public IRubyObject flatten(ThreadContext context, IRubyObject level) {
+        RubyArray ary = to_a();
+        ary.callMethod(context, "flatten!", level);
+        return ary;
+    }
+
     public boolean hasDefaultProc() {
         return (flags & PROCDEFAULT_HASH_F) != 0;
     }
