@@ -26,6 +26,7 @@
  * Copyright (C) 2006 Michael Studman <codehaus@michaelstudman.com>
  * Copyright (C) 2006 Miguel Covarrubias <mlcovarrubias@gmail.com>
  * Copyright (C) 2007 Nick Sieger <nicksieger@gmail.com>
+ * Copyright (C) 2008 Joseph LaFata <joe@quibb.org>
  * 
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -44,6 +45,7 @@ package org.jruby;
 import java.io.ByteArrayOutputStream;
 import java.math.BigInteger;
 import java.util.ArrayList;
+import java.util.Random;
 
 import static org.jruby.RubyEnumerator.enumeratorize;
 import static org.jruby.anno.FrameField.*;
@@ -61,7 +63,6 @@ import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.CallType;
 import org.jruby.runtime.Frame;
-import org.jruby.runtime.MethodIndex;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import static org.jruby.runtime.Visibility.*;
@@ -274,7 +275,7 @@ public class RubyKernel {
         }else if(object.isNil()){
             throw recv.getRuntime().newTypeError("can't convert nil into Float");
         } else {
-            RubyFloat rFloat = (RubyFloat)TypeConverter.convertToType(object, recv.getRuntime().getFloat(), MethodIndex.TO_F, "to_f");
+            RubyFloat rFloat = (RubyFloat)TypeConverter.convertToType(object, recv.getRuntime().getFloat(), "to_f");
             if (Double.isNaN(rFloat.getDoubleValue())) throw recv.getRuntime().newArgumentError("invalid value for Float()");
             return rFloat;
         }
@@ -293,14 +294,14 @@ public class RubyKernel {
             return RubyNumeric.str2inum(context.getRuntime(),(RubyString)object,0,true);
         }
         
-        IRubyObject tmp = TypeConverter.convertToType(object, context.getRuntime().getInteger(), MethodIndex.TO_INT, "to_int", false);
-        if (tmp.isNil()) return object.convertToInteger(MethodIndex.TO_I, "to_i");
+        IRubyObject tmp = TypeConverter.convertToType(object, context.getRuntime().getInteger(), "to_int", false);
+        if (tmp.isNil()) return object.convertToInteger("to_i");
         return tmp;
     }
 
     @JRubyMethod(name = "String", required = 1, module = true, visibility = PRIVATE)
     public static IRubyObject new_string(ThreadContext context, IRubyObject recv, IRubyObject object) {
-        return TypeConverter.convertToType(object, context.getRuntime().getString(), MethodIndex.TO_S, "to_s");
+        return TypeConverter.convertToType(object, context.getRuntime().getString(), "to_s");
     }
 
     @JRubyMethod(name = "p", rest = true, module = true, visibility = PRIVATE)
@@ -383,7 +384,7 @@ public class RubyKernel {
     }
 
     @JRubyMethod(name = "readlines", optional = 1, module = true, visibility = PRIVATE)
-    public static RubyArray readlines(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
+    public static IRubyObject readlines(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         return RubyArgsFile.readlines(context, context.getRuntime().getGlobalVariables().get("$<"), args);
     }
 
@@ -649,11 +650,13 @@ public class RubyKernel {
         long startTime = System.currentTimeMillis();
         
         RubyThread rubyThread = context.getThread();
-        
+
+        // Spurious wakeup-loop
         do {
             long loopStartTime = System.currentTimeMillis();
             try {
-                rubyThread.sleep(milliseconds);
+                // We break if we know this sleep was explicitly woken up/interrupted
+                if (!rubyThread.sleep(milliseconds)) break;
             } catch (InterruptedException iExcptn) {
             }
             milliseconds -= (System.currentTimeMillis() - loopStartTime);
@@ -726,8 +729,13 @@ public class RubyKernel {
         return localVariables;
     }
 
-    @JRubyMethod(name = "binding", module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "binding", module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_8)
     public static RubyBinding binding(ThreadContext context, IRubyObject recv, Block block) {
+        return RubyBinding.newBinding(context.getRuntime(), recv);
+    }
+
+    @JRubyMethod(name = "binding", module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_9)
+    public static RubyBinding binding_1_9(ThreadContext context, IRubyObject recv, Block block) {
         return RubyBinding.newBinding(context.getRuntime());
     }
 
@@ -978,7 +986,7 @@ public class RubyKernel {
         if (var.charAt(0) != '$') return context.getRuntime().getNil();
         if (args.length == 1) proc = RubyProc.newProc(context.getRuntime(), block, Block.Type.PROC);
         if (args.length == 2) {
-            proc = (RubyProc)TypeConverter.convertToType(args[1], context.getRuntime().getProc(), 0, "to_proc", true);
+            proc = (RubyProc)TypeConverter.convertToType(args[1], context.getRuntime().getProc(), "to_proc", true);
         }
         
         context.getRuntime().getGlobalVariables().setTraceVar(var, proc);
@@ -1167,63 +1175,64 @@ public class RubyKernel {
         
         runtime.getGlobalVariables().set("$?", RubyProcess.RubyStatus.newProcessStatus(runtime, resultCode));
         
-        return RubyString.newString(runtime, output.toByteArray());
+        return RubyString.newStringNoCopy(runtime, output.toByteArray());
     }
     
-    @JRubyMethod(name = "srand", optional = 1, module = true, visibility = PRIVATE)
-    public static RubyInteger srand(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
+    @JRubyMethod(name = "srand", module = true, visibility = PRIVATE)
+    public static RubyInteger srand(ThreadContext context, IRubyObject recv) {
         Ruby runtime = context.getRuntime();
-        long oldRandomSeed = runtime.getRandomSeed();
 
-        if (args.length > 0) {
-            RubyInteger integerSeed = args[0].convertToInteger(MethodIndex.TO_INT, "to_int");
-            runtime.setRandomSeed(integerSeed.getLongValue());
-        } else {
-            // Not sure how well this works, but it works much better than
-            // just currentTimeMillis by itself.
-            runtime.setRandomSeed(System.currentTimeMillis() ^
-              recv.hashCode() ^ runtime.incrementRandomSeedSequence() ^
-              runtime.getRandom().nextInt(Math.max(1, Math.abs((int)runtime.getRandomSeed()))));
-        }
+        // Not sure how well this works, but it works much better than
+        // just currentTimeMillis by itself.
+        runtime.setRandomSeed(System.currentTimeMillis() ^
+               recv.hashCode() ^ runtime.incrementRandomSeedSequence() ^
+               runtime.getRandom().nextInt(Math.max(1, Math.abs((int)runtime.getRandomSeed()))));
+
+        long oldRandomSeed = runtime.getRandomSeed();
+        runtime.getRandom().setSeed(runtime.getRandomSeed());
+        return runtime.newFixnum(oldRandomSeed);
+    }
+    
+    @JRubyMethod(name = "srand", module = true, visibility = PRIVATE)
+    public static RubyInteger srand(ThreadContext context, IRubyObject recv, IRubyObject arg) {
+        RubyInteger integerSeed = arg.convertToInteger("to_int");
+        Ruby runtime = context.getRuntime();
+
+        long oldRandomSeed = runtime.getRandomSeed();
+        runtime.setRandomSeed(integerSeed.getLongValue());
+
         runtime.getRandom().setSeed(runtime.getRandomSeed());
         return runtime.newFixnum(oldRandomSeed);
     }
 
-    @JRubyMethod(name = "rand", optional = 1, module = true, visibility = PRIVATE)
-    public static RubyNumeric rand(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
+    @JRubyMethod(name = "rand", module = true, visibility = PRIVATE)
+    public static RubyNumeric rand(ThreadContext context, IRubyObject recv) {
         Ruby runtime = context.getRuntime();
-        long ceil;
-        if (args.length == 0) {
-            ceil = 0;
-        } else if (args.length == 1) {
-            if (args[0] instanceof RubyBignum) {
-                byte[] bigCeilBytes = ((RubyBignum) args[0]).getValue().toByteArray();
-                BigInteger bigCeil = new BigInteger(bigCeilBytes).abs();
-                
-                byte[] randBytes = new byte[bigCeilBytes.length];
-                runtime.getRandom().nextBytes(randBytes);
-                
-                BigInteger result = new BigInteger(randBytes).abs().mod(bigCeil);
-                
-                return new RubyBignum(runtime, result); 
-            }
-             
-            RubyInteger integerCeil = (RubyInteger)RubyKernel.new_integer(context, recv, args[0]); 
-            ceil = Math.abs(integerCeil.getLongValue());
-        } else {
-            throw runtime.newArgumentError("wrong # of arguments(" + args.length + " for 1)");
+        return RubyFloat.newFloat(runtime, runtime.getRandom().nextDouble());
+    }
+
+    @JRubyMethod(name = "rand", module = true, visibility = PRIVATE)
+    public static RubyNumeric rand(ThreadContext context, IRubyObject recv, IRubyObject arg) {
+        Ruby runtime = context.getRuntime();
+        Random random = runtime.getRandom();
+
+        if (arg instanceof RubyBignum) {
+            byte[] bigCeilBytes = ((RubyBignum) arg).getValue().toByteArray();
+            BigInteger bigCeil = new BigInteger(bigCeilBytes).abs();
+            byte[] randBytes = new byte[bigCeilBytes.length];
+            random.nextBytes(randBytes);
+            BigInteger result = new BigInteger(randBytes).abs().mod(bigCeil);
+            return new RubyBignum(runtime, result); 
         }
 
-        if (ceil == 0) {
-            return RubyFloat.newFloat(runtime, runtime.getRandom().nextDouble()); 
-        }
-        if (ceil > Integer.MAX_VALUE) {
-            return runtime.newFixnum(Math.abs(runtime.getRandom().nextLong()) % ceil);
-        }
-            
-        return runtime.newFixnum(runtime.getRandom().nextInt((int) ceil));
+        RubyInteger integerCeil = (RubyInteger)RubyKernel.new_integer(context, recv, arg); 
+        long ceil = Math.abs(integerCeil.getLongValue());
+        if (ceil == 0) return RubyFloat.newFloat(runtime, random.nextDouble());
+        if (ceil > Integer.MAX_VALUE) return runtime.newFixnum(Math.abs(random.nextLong()) % ceil);
+
+        return runtime.newFixnum(random.nextInt((int) ceil));
     }
-    
+
     @JRubyMethod(name = "syscall", required = 1, optional = 9, module = true, visibility = PRIVATE)
     public static IRubyObject syscall(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         throw context.getRuntime().newNotImplementedError("Kernel#syscall is not implemented in JRuby");
