@@ -31,48 +31,24 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby;
 
-import org.jruby.runtime.CallbackFactory;
+import org.jruby.anno.JRubyMethod;
+import org.jruby.anno.JRubyModule;
+import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
 
+@JRubyModule(name="Math")
 public class RubyMath {
     /** Create the Math module and add it to the Ruby runtime.
      * 
      */
     public static RubyModule createMathModule(Ruby runtime) {
         RubyModule result = runtime.defineModule("Math");
-        CallbackFactory callbackFactory = runtime.callbackFactory(RubyMath.class);
+        runtime.setMath(result);
         
         result.defineConstant("E", RubyFloat.newFloat(runtime, Math.E));
         result.defineConstant("PI", RubyFloat.newFloat(runtime, Math.PI));
-
-        result.defineFastModuleFunction("atan2", callbackFactory.getFastSingletonMethod("atan2", RubyKernel.IRUBY_OBJECT, RubyKernel.IRUBY_OBJECT));
-        result.defineFastModuleFunction("cos", callbackFactory.getFastSingletonMethod("cos", RubyKernel.IRUBY_OBJECT));
-        result.defineFastModuleFunction("sin", callbackFactory.getFastSingletonMethod("sin", RubyKernel.IRUBY_OBJECT));
-        result.defineFastModuleFunction("tan", callbackFactory.getFastSingletonMethod("tan", RubyKernel.IRUBY_OBJECT));
-
-        result.defineFastModuleFunction("acos", callbackFactory.getFastSingletonMethod("acos", RubyKernel.IRUBY_OBJECT));        
-        result.defineFastModuleFunction("asin", callbackFactory.getFastSingletonMethod("asin", RubyKernel.IRUBY_OBJECT));        
-        result.defineFastModuleFunction("atan", callbackFactory.getFastSingletonMethod("atan", RubyKernel.IRUBY_OBJECT));        
-
-        result.defineFastModuleFunction("cosh", callbackFactory.getFastSingletonMethod("cosh", RubyKernel.IRUBY_OBJECT));        
-        result.defineFastModuleFunction("sinh", callbackFactory.getFastSingletonMethod("sinh", RubyKernel.IRUBY_OBJECT));        
-        result.defineFastModuleFunction("tanh", callbackFactory.getFastSingletonMethod("tanh", RubyKernel.IRUBY_OBJECT));        
         
-        result.defineFastModuleFunction("acosh", callbackFactory.getFastSingletonMethod("acosh", RubyKernel.IRUBY_OBJECT));        
-        result.defineFastModuleFunction("asinh", callbackFactory.getFastSingletonMethod("asinh", RubyKernel.IRUBY_OBJECT));
-        result.defineFastModuleFunction("atanh", callbackFactory.getFastSingletonMethod("atanh", RubyKernel.IRUBY_OBJECT));
-        
-        result.defineFastModuleFunction("exp", callbackFactory.getFastSingletonMethod("exp", RubyKernel.IRUBY_OBJECT));
-        result.defineFastModuleFunction("log", callbackFactory.getFastSingletonMethod("log", RubyKernel.IRUBY_OBJECT));
-        result.defineFastModuleFunction("log10", callbackFactory.getFastSingletonMethod("log10", RubyKernel.IRUBY_OBJECT));
-        result.defineFastModuleFunction("sqrt", callbackFactory.getFastSingletonMethod("sqrt", RubyKernel.IRUBY_OBJECT));
-        
-        result.defineFastModuleFunction("frexp", callbackFactory.getFastSingletonMethod("frexp", RubyKernel.IRUBY_OBJECT));
-        result.defineFastModuleFunction("ldexp", callbackFactory.getFastSingletonMethod("ldexp", RubyKernel.IRUBY_OBJECT, RubyKernel.IRUBY_OBJECT));
-        
-        result.defineFastModuleFunction("hypot", callbackFactory.getFastSingletonMethod("hypot", RubyKernel.IRUBY_OBJECT, RubyKernel.IRUBY_OBJECT));        
-        result.defineFastModuleFunction("erf", callbackFactory.getFastSingletonMethod("erf", RubyKernel.IRUBY_OBJECT));        
-        result.defineFastModuleFunction("erfc", callbackFactory.getFastSingletonMethod("erfc", RubyKernel.IRUBY_OBJECT));
+        result.defineAnnotatedMethods(RubyMath.class);
 
         return result;
     }
@@ -104,27 +80,32 @@ public class RubyMath {
         return (y < 0.0) ? -abs : abs;
     }
     
+    @JRubyMethod(name = "atan2", required = 2, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat atan2(IRubyObject recv, IRubyObject x, IRubyObject y) {
         double valuea = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         double valueb = ((RubyFloat)RubyKernel.new_float(recv,y)).getDoubleValue();
         return RubyFloat.newFloat(recv.getRuntime(), Math.atan2(valuea, valueb));
     }
 
+    @JRubyMethod(name = "cos", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat cos(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         return RubyFloat.newFloat(recv.getRuntime(),Math.cos(value));
     }
 
+    @JRubyMethod(name = "sin", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat sin(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         return RubyFloat.newFloat(recv.getRuntime(),Math.sin(value));
     }
 
+    @JRubyMethod(name = "tan", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat tan(IRubyObject recv,  IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         return RubyFloat.newFloat(recv.getRuntime(),Math.tan(value));
     }
     
+    @JRubyMethod(name = "asin", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat asin(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         double result = Math.asin(value);
@@ -132,6 +113,7 @@ public class RubyMath {
         return RubyFloat.newFloat(recv.getRuntime(),result);
     }
 
+    @JRubyMethod(name = "acos", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat acos(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         double result = Math.acos(value);  
@@ -139,25 +121,31 @@ public class RubyMath {
         return RubyFloat.newFloat(recv.getRuntime(), result);
     }
     
+    @JRubyMethod(name = "atan", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat atan(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         return RubyFloat.newFloat(recv.getRuntime(),Math.atan(value));
     }
 
+    @JRubyMethod(name = "cosh", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat cosh(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         return RubyFloat.newFloat(recv.getRuntime(),(Math.exp(value) + Math.exp(-value)) / 2.0);
     }    
 
+    @JRubyMethod(name = "sinh", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat sinh(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         return RubyFloat.newFloat(recv.getRuntime(),(Math.exp(value) - Math.exp(-value)) / 2.0);
     }
     
+    @JRubyMethod(name = "tanh", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat tanh(IRubyObject recv, IRubyObject x) {
-        return RubyFloat.newFloat(recv.getRuntime(), sinh(recv, x).getDoubleValue() / cosh(recv, x).getDoubleValue());
+        double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
+        return RubyFloat.newFloat(recv.getRuntime(), Math.tanh(value));
     }          
     
+    @JRubyMethod(name = "acosh", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat acosh(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         double result;
@@ -176,28 +164,29 @@ public class RubyMath {
     
     private static final double ASINH_COEF[] = {
         -.12820039911738186343372127359268e+0,
-       -.58811761189951767565211757138362e-1,
-       .47274654322124815640725249756029e-2,
-       -.49383631626536172101360174790273e-3,
-       .58506207058557412287494835259321e-4,
-       -.74669983289313681354755069217188e-5,
-       .10011693583558199265966192015812e-5,
-       -.13903543858708333608616472258886e-6,
-       .19823169483172793547317360237148e-7,
-       -.28847468417848843612747272800317e-8,
-       .42672965467159937953457514995907e-9,
-       -.63976084654366357868752632309681e-10,
-       .96991686089064704147878293131179e-11,
-       -.14844276972043770830246658365696e-11,
-       .22903737939027447988040184378983e-12,
-       -.35588395132732645159978942651310e-13,
-       .55639694080056789953374539088554e-14,
-       -.87462509599624678045666593520162e-15,
-       .13815248844526692155868802298129e-15,
-       -.21916688282900363984955142264149e-16,
-       .34904658524827565638313923706880e-17
+        -.58811761189951767565211757138362e-1,
+        .47274654322124815640725249756029e-2,
+        -.49383631626536172101360174790273e-3,
+        .58506207058557412287494835259321e-4,
+        -.74669983289313681354755069217188e-5,
+        .10011693583558199265966192015812e-5,
+        -.13903543858708333608616472258886e-6,
+        .19823169483172793547317360237148e-7,
+        -.28847468417848843612747272800317e-8,
+        .42672965467159937953457514995907e-9,
+        -.63976084654366357868752632309681e-10,
+        .96991686089064704147878293131179e-11,
+        -.14844276972043770830246658365696e-11,
+        .22903737939027447988040184378983e-12,
+        -.35588395132732645159978942651310e-13,
+        .55639694080056789953374539088554e-14,
+        -.87462509599624678045666593520162e-15,
+        .13815248844526692155868802298129e-15,
+        -.21916688282900363984955142264149e-16,
+        .34904658524827565638313923706880e-17
     };      
     
+    @JRubyMethod(name = "asinh", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat asinh(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         double  y = Math.abs(value);
@@ -210,13 +199,12 @@ public class RubyMath {
         } else if (y <= 1.0) {          
             result = value * (1.0 + chebylevSerie(2.0 * value * value - 1.0, ASINH_COEF));
         } else if (y < 94906265.62) {
-            result = Math.log(y + Math.sqrt(y * y + 1.0));
+            result = Math.log(value + Math.sqrt(value * value + 1.0));
         } else {    
             result = 0.69314718055994530941723212145818 + Math.log(y);
+            if (value < 0) result *= -1;
         }
-        if (value < 0.0) {
-            result = -result;
-        }
+
         return RubyFloat.newFloat(recv.getRuntime(),result);        
     }
     
@@ -239,6 +227,7 @@ public class RubyMath {
         .4497954546494931083083327624533e-18
     };    
     
+    @JRubyMethod(name = "atanh", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat atanh(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         double  y = Math.abs(value);
@@ -262,6 +251,7 @@ public class RubyMath {
         return RubyFloat.newFloat(recv.getRuntime(),result);        
     }
     
+    @JRubyMethod(name = "exp", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat exp(IRubyObject recv, IRubyObject exponent) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,exponent)).getDoubleValue();
         return RubyFloat.newFloat(recv.getRuntime(),Math.exp(value));
@@ -270,6 +260,7 @@ public class RubyMath {
     /** Returns the natural logarithm of x.
      * 
      */
+    @JRubyMethod(name = "log", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat log(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         double result = Math.log(value);
@@ -280,6 +271,7 @@ public class RubyMath {
     /** Returns the base 10 logarithm of x.
      * 
      */
+    @JRubyMethod(name = "log10", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat log10(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         double result =  Math.log(value) / Math.log(10);
@@ -287,6 +279,7 @@ public class RubyMath {
         return RubyFloat.newFloat(recv.getRuntime(),result);
     }
 
+    @JRubyMethod(name = "sqrt", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat sqrt(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         double result;
@@ -301,6 +294,7 @@ public class RubyMath {
         return RubyFloat.newFloat(recv.getRuntime(), result);
     }
     
+    @JRubyMethod(name = "hypot", required = 2, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat hypot(IRubyObject recv, IRubyObject x, IRubyObject y) {
         double valuea = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue(); 
         double valueb = ((RubyFloat)RubyKernel.new_float(recv,y)).getDoubleValue();
@@ -325,12 +319,13 @@ public class RubyMath {
      * Where mantissa is in the range of [.5, 1)
      *
      */
+    @JRubyMethod(name = "frexp", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyArray frexp(IRubyObject recv, IRubyObject other) {
         double mantissa = ((RubyFloat)RubyKernel.new_float(recv,other)).getDoubleValue();
         short sign = 1;
         long exponent = 0;
 
-        if (mantissa != 0.0) {
+        if (!Double.isInfinite(mantissa) && mantissa != 0.0) {
             // Make mantissa same sign so we only have one code path.
             if (mantissa < 0) {
                 mantissa = -mantissa;
@@ -352,6 +347,7 @@ public class RubyMath {
     /*
      * r = x * 2 ** y
      */
+    @JRubyMethod(name = "ldexp", required = 2, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat ldexp(IRubyObject recv, IRubyObject mantissa, IRubyObject exponent) {
         double mantissaValue = ((RubyFloat)RubyKernel.new_float(recv, mantissa)).getDoubleValue();
         return RubyFloat.newFloat(recv.getRuntime(),mantissaValue * Math.pow(2.0, RubyNumeric.num2int(exponent)));
@@ -374,6 +370,7 @@ public class RubyMath {
          -.126124551191552258324954248533e-18
     };
     
+    @JRubyMethod(name = "erf", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat erf(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
 
@@ -454,6 +451,7 @@ public class RubyMath {
          .194744338223207851429197867821e-18
     };
         
+    @JRubyMethod(name = "erfc", required = 1, module = true, visibility = Visibility.PRIVATE)
     public static RubyFloat erfc(IRubyObject recv, IRubyObject x) {
         double value = ((RubyFloat)RubyKernel.new_float(recv,x)).getDoubleValue();
         double  result;
