@@ -41,14 +41,16 @@ import static org.jruby.RubyEnumerator.enumeratorize;
 import java.io.IOException;
 import java.util.List;
 
+import org.jcodings.Encoding;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
+import org.jruby.exceptions.JumpException;
 import org.jruby.exceptions.RaiseException;
+import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.BlockCallback;
 import org.jruby.runtime.CallBlock;
-import org.jruby.runtime.MethodIndex;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ObjectMarshal;
 import org.jruby.runtime.ThreadContext;
@@ -57,6 +59,7 @@ import org.jruby.runtime.builtin.Variable;
 import org.jruby.runtime.component.VariableEntry;
 import org.jruby.runtime.marshal.MarshalStream;
 import org.jruby.runtime.marshal.UnmarshalStream;
+import org.jruby.util.ByteList;
 import org.jruby.util.TypeConverter;
 
 /**
@@ -117,7 +120,8 @@ public class RubyRange extends RubyObject {
         return range;
     }
 
-    protected void copySpecialInstanceVariables(IRubyObject clone) {
+    @Override
+    public void copySpecialInstanceVariables(IRubyObject clone) {
         RubyRange range = (RubyRange)clone;
         range.begin = begin;
         range.end = end;
@@ -150,12 +154,40 @@ public class RubyRange extends RubyObject {
         if (len < 0) len = 0;
 
         return new long[]{beg, len};
-    }    
+    }
+
+    final int[] begLenInt(int len, int err){
+        int beg = RubyNumeric.num2int(this.begin);
+        int end = RubyNumeric.num2int(this.end);
+
+        if (beg < 0) {
+            beg += len;
+            if (beg < 0) {
+                if (err != 0) throw getRuntime().newRangeError(beg + ".." + (isExclusive ? "." : "") + end + " out of range");
+                return null;
+            }
+        }
+
+        if (err == 0 || err == 2) {
+            if (beg > len) {
+                if (err != 0) throw getRuntime().newRangeError(beg + ".." + (isExclusive ? "." : "") + end + " out of range");
+                return null;
+            }
+            if (end > len) end = len;
+        }
+
+        if (end < 0) end += len;
+        if (!isExclusive) end++;
+        len = end - beg;
+        if (len < 0) len = 0;
+
+        return new int[]{beg, len};
+    }
 
     private void init(ThreadContext context, IRubyObject begin, IRubyObject end, boolean isExclusive) {
         if (!(begin instanceof RubyFixnum && end instanceof RubyFixnum)) {
             try {
-                IRubyObject result = begin.callMethod(context, MethodIndex.OP_SPACESHIP, "<=>", end);
+                IRubyObject result = begin.callMethod(context, "<=>", end);
                 if (result.isNil()) throw getRuntime().newArgumentError("bad value for range");
             } catch (RaiseException re) {
                 throw getRuntime().newArgumentError("bad value for range");
@@ -277,7 +309,7 @@ public class RubyRange extends RubyObject {
             if (iter instanceof RubyFixnum) {
                 iter = RubyFixnum.newFixnum(context.getRuntime(), ((RubyFixnum)iter).getLongValue() - 1);                
             } else {
-                iter = iter.callMethod(context, MethodIndex.OP_MINUS, "-", RubyFixnum.one(context.getRuntime()));
+                iter = iter.callMethod(context, "-", RubyFixnum.one(context.getRuntime()));
             }
             if (iter == RubyFixnum.zero(context.getRuntime())) {
                 block.yield(context, arg);
@@ -287,13 +319,13 @@ public class RubyRange extends RubyObject {
     }
 
     private IRubyObject rangeLt(ThreadContext context, IRubyObject a, IRubyObject b) {
-        IRubyObject result = a.callMethod(context, MethodIndex.OP_SPACESHIP, "<=>", b);
+        IRubyObject result = a.callMethod(context, "<=>", b);
         if (result.isNil()) return null;
         return RubyComparable.cmpint(context, result, a, b) < 0 ? getRuntime().getTrue() : null;
     }
 
     private IRubyObject rangeLe(ThreadContext context, IRubyObject a, IRubyObject b) {
-        IRubyObject result = a.callMethod(context, MethodIndex.OP_SPACESHIP, "<=>", b);
+        IRubyObject result = a.callMethod(context, "<=>", b);
         if (result.isNil()) return null;
         int c = RubyComparable.cmpint(context, result, a, b);
         if (c == 0) return RubyFixnum.zero(getRuntime());
@@ -334,7 +366,7 @@ public class RubyRange extends RubyObject {
             for (int i = 0; i < size; i++) {
                 array[i] = RubyFixnum.newFixnum(runtime, base + i);
             }
-            return RubyArray.newArray(runtime, array);
+            return RubyArray.newArrayNoCopy(runtime, array);
         } else {
             return RubyEnumerable.to_a(context, this);
         }
@@ -408,18 +440,15 @@ public class RubyRange extends RubyObject {
                     throw runtime.newArgumentError("step can't be 0");
                 }
                 final String method;
-                final int methodIndex;
                 if (isExclusive) {
                     method = "<";
-                    methodIndex = MethodIndex.OP_LT;
                 } else {
                     method = "<=";
-                    methodIndex = MethodIndex.OP_LE;
                 }
                 IRubyObject beg = begin;
-                while (beg.callMethod(context, methodIndex, method, end).isTrue()) {
+                while (beg.callMethod(context, method, end).isTrue()) {
                     block.yield(context, beg);
-                    beg = beg.callMethod(context, MethodIndex.OP_PLUS, "+", step);
+                    beg = beg.callMethod(context, "+", step);
                 }
             } else {
                 if (unit == 0) throw runtime.newArgumentError("step can't be 0");
@@ -442,7 +471,7 @@ public class RubyRange extends RubyObject {
         return block.isGiven() ? step(context, step, block) : enumeratorize(context.getRuntime(), this, "step", step);
     }
 
-    @JRubyMethod(name = {"include?", "member?", "==="}, required = 1)
+    @JRubyMethod(name = {"include?", "member?", "==="}, required = 1, compat = CompatVersion.RUBY1_8)
     public RubyBoolean include_p(ThreadContext context, IRubyObject obj) {
         if (rangeLe(context, begin, obj) != null) {
             if (isExclusive) {
@@ -454,6 +483,111 @@ public class RubyRange extends RubyObject {
         return context.getRuntime().getFalse();
     }
 
+    @JRubyMethod(name = {"include?", "member?"}, frame = true, compat = CompatVersion.RUBY1_9)
+    public IRubyObject include_p19(ThreadContext context, IRubyObject obj) {
+        Ruby runtime = context.getRuntime();
+        if (begin instanceof RubyNumeric || end instanceof RubyNumeric ||
+                !TypeConverter.convertToTypeWithCheck(begin, runtime.getInteger(), "to_int").isNil() ||
+                !TypeConverter.convertToTypeWithCheck(end, runtime.getInteger(), "to_int").isNil()) {
+            if (rangeLe(context, begin, obj) != null) {
+                if (isExclusive) {
+                    if (rangeLt(context, obj, end) != null) return runtime.getTrue();
+                } else {
+                    if (rangeLe(context, obj, end) != null) return runtime.getTrue();
+                }
+            }
+            return runtime.getFalse();
+        } else if (begin instanceof RubyString && end instanceof RubyString &&
+                ((RubyString)begin).getByteList().realSize == 1 &&
+                ((RubyString)end).getByteList().realSize == 1) {
+            if (obj.isNil()) return runtime.getFalse();
+            if (obj instanceof RubyString) {
+                ByteList Vbytes = ((RubyString)obj).getByteList();
+                if (Vbytes.realSize != 1) return runtime.getFalse();
+                int v = Vbytes.bytes[Vbytes.begin] & 0xff;
+                ByteList Bbytes = ((RubyString)begin).getByteList();
+                int b = Bbytes.bytes[Bbytes.begin] & 0xff;
+                ByteList Ebytes = ((RubyString)end).getByteList();
+                int e = Ebytes.bytes[Ebytes.begin] & 0xff;
+                if (Encoding.isAscii(v) && Encoding.isAscii(b) && Encoding.isAscii(e)) {
+                    if ((b <= v && v < e) || (!isExclusive && v == e)) return runtime.getTrue();
+                    return runtime.getFalse();
+                }
+            }
+        }
+        return RuntimeHelpers.invokeSuper(context, this, obj, Block.NULL_BLOCK);
+    }
+
+    @JRubyMethod(name = "===", compat = CompatVersion.RUBY1_9)
+    public IRubyObject eqq_p19(ThreadContext context, IRubyObject obj) {
+        return callMethod(context, "include?", obj);
+    }
+
+    @JRubyMethod(name = "cover?", compat = CompatVersion.RUBY1_9)
+    public IRubyObject cover_p(ThreadContext context, IRubyObject obj) {
+        return include_p(context, obj); // 1.8 "include?"
+    }
+
+    @JRubyMethod(name = "min", frame = true, compat = CompatVersion.RUBY1_9)
+    public IRubyObject min(ThreadContext context, Block block) {
+        if (block.isGiven()) {
+            return RuntimeHelpers.invokeSuper(context, this, block);
+        } else {
+            int c = RubyComparable.cmpint(context, begin.callMethod(context, "<=>", end), begin, end);
+            if (c > 0 || (c == 0 && isExclusive)) return context.getRuntime().getNil();
+            return begin;
+        }
+    }
+
+    @JRubyMethod(name = "max", frame = true, compat = CompatVersion.RUBY1_9)
+    public IRubyObject max(ThreadContext context, Block block) {
+        if (block.isGiven() || isExclusive && !(end instanceof RubyInteger)) {
+            return RuntimeHelpers.invokeSuper(context, this, block);
+        } else {
+            int c = RubyComparable.cmpint(context, begin.callMethod(context, "<=>", end), begin, end);
+            Ruby runtime = context.getRuntime();
+            if (isExclusive) {
+                if (c == 0) return runtime.getNil();
+                if (end instanceof RubyFixnum) return RubyFixnum.newFixnum(runtime, ((RubyFixnum)end).getLongValue() - 1);
+                return end.callMethod(context, "-", RubyFixnum.one(runtime));
+            }
+            return end;
+        }
+    }
+
+    @JRubyMethod(name = "first", compat = CompatVersion.RUBY1_9)
+    public IRubyObject first(ThreadContext context) {
+        return begin;
+    }
+
+    @JRubyMethod(name = "first", compat = CompatVersion.RUBY1_9)
+    public IRubyObject first(ThreadContext context, IRubyObject arg) {
+        final Ruby runtime = context.getRuntime();
+        final int num = RubyNumeric.num2int(arg);
+        final RubyArray result = runtime.newArray(num);
+        try {
+            RubyEnumerable.callEach(runtime, context, this, new BlockCallback() {
+                int n = num;
+                public IRubyObject call(ThreadContext ctx, IRubyObject[] largs, Block blk) {
+                    if (n-- <= 0) throw JumpException.SPECIAL_JUMP;
+                    result.append(largs[0]);
+                    return runtime.getNil();
+                }
+            });
+        } catch (JumpException.SpecialJump sj) {}
+        return result;
+    }
+
+    @JRubyMethod(name = "last", compat = CompatVersion.RUBY1_9)
+    public IRubyObject last(ThreadContext context) {
+        return end;
+    }
+
+    @JRubyMethod(name = "last", compat = CompatVersion.RUBY1_9)
+    public IRubyObject last(ThreadContext context, IRubyObject arg) {
+        return ((RubyArray)RubyKernel.new_array(context, this, this)).last(arg);
+    }
+
     private static final ObjectMarshal RANGE_MARSHAL = new ObjectMarshal() {
         public void marshalTo(Ruby runtime, Object obj, RubyClass type,
                               MarshalStream marshalStream) throws IOException {
