@@ -63,32 +63,32 @@ import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.jruby.runtime.encoding.EncodingCapable;
 import org.jruby.runtime.marshal.MarshalStream;
 import org.jruby.runtime.marshal.UnmarshalStream;
 import org.jruby.util.ByteList;
 import org.jruby.util.KCode;
 import org.jruby.util.TypeConverter;
 
-/**
- *
- */
 @JRubyClass(name="Regexp")
-public class RubyRegexp extends RubyObject implements ReOptions, WarnCallback {
+public class RubyRegexp extends RubyObject implements ReOptions, WarnCallback, EncodingCapable {
     private KCode kcode;
     private Regex pattern;
     private ByteList str;
 
-    private static final int REGEXP_LITERAL_F =     1 << 11;
-    private static final int REGEXP_KCODE_DEFAULT = 1 << 12;
+    private static final int REGEXP_LITERAL_F     = USER1_F;
+    private static final int REGEXP_KCODE_DEFAULT = USER2_F;
+    private static final int REGEXP_KCODE_FIXED   = USER3_F;
+    private static final int REGEXP_ENCODING_NONE = USER4_F;
 
     public void setLiteral() {
         flags |= REGEXP_LITERAL_F;
     }
-    
+
     public void clearLiteral() {
         flags &= ~REGEXP_LITERAL_F;
     }
-    
+
     public boolean isLiteral() {
         return (flags & REGEXP_LITERAL_F) != 0;
     }
@@ -96,19 +96,47 @@ public class RubyRegexp extends RubyObject implements ReOptions, WarnCallback {
     public void setKCodeDefault() {
         flags |= REGEXP_KCODE_DEFAULT;
     }
-    
+
     public void clearKCodeDefault() {
         flags &= ~REGEXP_KCODE_DEFAULT;
     }
-    
+
     public boolean isKCodeDefault() {
         return (flags & REGEXP_KCODE_DEFAULT) != 0;
     }
 
+    public void setKCodeFixed() {
+        flags |= REGEXP_KCODE_FIXED;
+    }
+
+    public void clearKCodeFixed() {
+        flags &= ~REGEXP_KCODE_FIXED;
+    }
+
+    public boolean isKCodeFixed() {
+        return (flags & REGEXP_KCODE_FIXED) != 0;
+    }
+
+    public void setEncodingNone() {
+        flags |= REGEXP_ENCODING_NONE;
+    }
+
+    public void clearEncodingNone() {
+        flags &= ~REGEXP_ENCODING_NONE;
+    }
+
+    public boolean isEncodingNone() {
+        return (flags & REGEXP_ENCODING_NONE) != 0;
+    }
+
     public KCode getKCode() {
         return kcode;
     }
 
+    public Encoding getEncoding() {
+        return pattern.getEncoding();
+    }
+
     private static Map<ByteList, Regex> getPatternCache() {
         Map<ByteList, Regex> cache = patternCache.get();
         if (cache == null) {
@@ -288,9 +316,13 @@ public class RubyRegexp extends RubyObject implements ReOptions, WarnCallback {
             ((pat.getUserOptions() & REGEX_QUOTED) != 0) == quote) { // cache hit
             pattern = pat;
         } else {
-            if (quote) bytes = quote(bytes, getRuntime().getKCode());
-            makeRegexp(bytes, bytes.begin, bytes.realSize, options & 0xf, kcode.getEncoding());
-            if (quote) pattern.setUserOptions(REGEX_QUOTED);
+            if (quote) {
+                ByteList quoted = quote(bytes, getRuntime().getKCode());
+                makeRegexp(quoted, quoted.begin, quoted.realSize, options & 0xf, kcode.getEncoding());
+                pattern.setUserOptions(REGEX_QUOTED);
+            } else {
+                makeRegexp(bytes, bytes.begin, bytes.realSize, options & 0xf, kcode.getEncoding());
+            }
             cache.put(bytes, pattern);
         }
 
@@ -515,6 +547,11 @@ public class RubyRegexp extends RubyObject implements ReOptions, WarnCallback {
         return re;
     }
 
+    @JRubyMethod(name = "try_convert", meta = true, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject try_convert(ThreadContext context, IRubyObject recv, IRubyObject args) {
+        return TypeConverter.convertToTypeWithCheck(args, context.getRuntime().getRegexp(), "to_regexp");
+    }
+
     @JRubyMethod(name = "options")
     public IRubyObject options() {
         return getRuntime().newFixnum(getOptions());
@@ -522,7 +559,7 @@ public class RubyRegexp extends RubyObject implements ReOptions, WarnCallback {
     
     /** rb_reg_search
      */
-    public int search(ThreadContext context, RubyString str, int pos, boolean reverse) {
+    public final int search(ThreadContext context, RubyString str, int pos, boolean reverse) {
         Ruby runtime = context.getRuntime();
         Frame frame = context.getCurrentRubyFrame();
 
@@ -540,19 +577,15 @@ public class RubyRegexp extends RubyObject implements ReOptions, WarnCallback {
 
         int realSize = value.realSize;
         int begin = value.begin;
-        int range = reverse ? -pos : realSize - pos;
 
         Matcher matcher = pattern.matcher(value.bytes, begin, begin + realSize);
-
-        int result = matcher.search(begin + pos, begin + pos + range, Option.NONE);
+        int result = matcher.search(begin + pos, begin + (reverse ? 0 : realSize), Option.NONE);
 
         if (result < 0) {
             frame.setBackRef(runtime.getNil());
-            return result;
+        } else {
+            updateBackRef(context, str, frame, matcher);
         }
-
-        updateBackRef(context, str, frame, matcher);
-
         return result;
     }
     
@@ -560,22 +593,22 @@ public class RubyRegexp extends RubyObject implements ReOptions, WarnCallback {
         Ruby runtime = context.getRuntime();
         IRubyObject backref = frame.getBackRef();
         final RubyMatchData match;
-        if (backref == null || backref.isNil() || ((RubyMatchData)backref).used()) {
+        if (backref.isNil() || ((RubyMatchData)backref).used()) {
             match = new RubyMatchData(runtime);
         } else {
             match = (RubyMatchData)backref;
             match.setTaint(runtime.getSafeLevel() >= 3);
         }
-        
+
         match.regs = matcher.getRegion(); // lazy, null when no groups defined
         match.begin = matcher.getBegin();
         match.end = matcher.getEnd();
-        
+
         match.str = (RubyString)str.strDup(runtime).freeze(context);
-        match.pattern = pattern;
+        match.regexp = this;
 
         frame.setBackRef(match);
-        
+
         match.infectBy(this);
         match.infectBy(str);
         return match;
@@ -1110,7 +1143,7 @@ public class RubyRegexp extends RubyObject implements ReOptions, WarnCallback {
         if (args.length == 0) {
             return newRegexp(recv.getRuntime(), ByteList.create("(?!)"), 0, false);
         } else if (args.length == 1) {
-            IRubyObject v = TypeConverter.convertToTypeWithCheck(args[0], recv.getRuntime().getRegexp(), 0, "to_regexp");
+            IRubyObject v = TypeConverter.convertToTypeWithCheck(args[0], recv.getRuntime().getRegexp(), "to_regexp");
             if (!v.isNil()) {
                 return v;
             } else {
@@ -1127,7 +1160,7 @@ public class RubyRegexp extends RubyObject implements ReOptions, WarnCallback {
                 if (0 < i) {
                     source.cat((byte)'|');
                 }
-                IRubyObject v = TypeConverter.convertToTypeWithCheck(args[i], recv.getRuntime().getRegexp(), 0, "to_regexp");
+                IRubyObject v = TypeConverter.convertToTypeWithCheck(args[i], recv.getRuntime().getRegexp(), "to_regexp");
                 if (!v.isNil()) {
                     if (!((RubyRegexp)v).isKCodeDefault()) {
                         if (kcode == null) {
@@ -1167,10 +1200,10 @@ public class RubyRegexp extends RubyObject implements ReOptions, WarnCallback {
      * 
      */
     @JRubyMethod(name = "names", compat = CompatVersion.RUBY1_9)
-    public IRubyObject names() {
+    public IRubyObject names(ThreadContext context) {
         if (pattern.numberOfNames() == 0) return getRuntime().newEmptyArray();
 
-        RubyArray ary = getRuntime().newArray(pattern.numberOfNames());
+        RubyArray ary = context.getRuntime().newArray(pattern.numberOfNames());
         for (Iterator<NameEntry> i = pattern.namedBackrefIterator(); i.hasNext();) {
             NameEntry e = i.next();
             ary.append(RubyString.newStringShared(getRuntime(), e.name, e.nameP, e.nameEnd - e.nameP));
@@ -1195,7 +1228,18 @@ public class RubyRegexp extends RubyObject implements ReOptions, WarnCallback {
             hash.fastASet(RubyString.newStringShared(getRuntime(), e.name, e.nameP, e.nameEnd - e.nameP).freeze(context), ary);
         }
         return hash;
-    }    
+    }
+
+    @JRubyMethod(name = "encoding", compat = CompatVersion.RUBY1_9)
+    public IRubyObject encoding(ThreadContext context) {
+        return context.getRuntime().getEncodingService().getEncoding(pattern.getEncoding());
+    }
+    
+    @JRubyMethod(name = "fixed_encoding?", compat = CompatVersion.RUBY1_9)
+    public IRubyObject fixed_encoding_p(ThreadContext context) {
+        Ruby runtime = context.getRuntime();
+        return isKCodeFixed() ? runtime.getTrue() : runtime.getFalse();
+    }
 
     public static RubyRegexp unmarshalFrom(UnmarshalStream input) throws java.io.IOException {
         RubyRegexp result = newRegexp(input.getRuntime(), input.unmarshalString(), input.unmarshalInt(), false);
