@@ -11,7 +11,7 @@
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
- * Copyright (C) 2006 Mirko Stocker <me@misto.ch>
+ * Copyright (C) 2006-2007 Mirko Stocker <me@misto.ch>
  * 
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -114,7 +114,7 @@ public class ReWriteVisitor implements NodeVisitor {
 	protected void printNewlineAndIndentation() {
 		if (config.hasHereDocument()) config.fetchHereDocument().print();
 
-		print('\n');
+		print(config.getFormatHelper().getLineDelimiter());
 		config.getIndentor().printIndentation(config.getOutput());
 	}
 
@@ -123,12 +123,12 @@ public class ReWriteVisitor implements NodeVisitor {
 	}
 
     private void printCommentsBefore(Node iVisited) {
-        
-		for (Iterator it = iVisited.getComments().iterator(); it.hasNext(); ) {
-			CommentNode n = (CommentNode) it.next();
-			if(getStartLine(n) < getStartLine(iVisited)) {
+
+        for (CommentNode n: iVisited.getComments()) {
+			if (getStartLine(n) < getStartLine(iVisited)) {
 				visitNode(n);
-				printNewlineAndIndentation();
+				print(n.getContent());
+                printNewlineAndIndentation();
 			}
 		}
 	}
@@ -136,11 +136,11 @@ public class ReWriteVisitor implements NodeVisitor {
 	protected boolean printCommentsAfter(Node iVisited) {
 		boolean hasComment = false;
         
-		for (Iterator it = iVisited.getComments().iterator(); it.hasNext(); ) {
-			CommentNode n = (CommentNode) it.next();
+        for (CommentNode n: iVisited.getComments()) {
 			if(getStartLine(n) >= getEndLine(iVisited)) {
 				print(' ');
 				visitNode(n);
+                print(n.getContent());
 				hasComment = true;
 			}
 		}
@@ -149,7 +149,7 @@ public class ReWriteVisitor implements NodeVisitor {
 	}
 	
 	public void visitNode(Node iVisited) {
-		if (iVisited == null) return;
+            if (iVisited == null || iVisited.isInvisible()) return;
         
         printCommentsBefore(iVisited);
 
@@ -238,9 +238,9 @@ public class ReWriteVisitor implements NodeVisitor {
 		leaveCall();
 	}
 
-	public void visitAndPrintWithSeparator(Iterator it) {
+	public void visitAndPrintWithSeparator(Iterator<Node> it) {
 		while (it.hasNext()) {
-			Node n = (Node) it.next();
+			Node n = it.next();
 			factory.createIgnoreCommentsReWriteVisitor().visitNode(n);
 			if (it.hasNext())
 				print(config.getFormatHelper().getListSeparator());
@@ -284,26 +284,22 @@ public class ReWriteVisitor implements NodeVisitor {
 	}
 
 
-	private ArrayList collectAllArguments(ArgsNode iVisited) {
-		ArrayList arguments = new ArrayList();
+	private ArrayList<Node> collectAllArguments(ArgsNode iVisited) {
+		ArrayList<Node> arguments = new ArrayList<Node>();
         
-		if (iVisited.getArgs() != null) arguments.addAll(iVisited.getArgs().childNodes());
-        
-		if (iVisited.getOptArgs() != null) arguments.addAll(iVisited.getOptArgs().childNodes());
-        
-		if (iVisited.getRestArg() > 0) {
-			arguments.add(new ConstNode(null, '*' + config.getLocalVariables().getLocalVariable(iVisited.getRestArg())));
+		if (iVisited.getPre() != null) arguments.addAll(iVisited.getPre().childNodes());
+        if (iVisited.getOptArgs() != null) arguments.addAll(iVisited.getOptArgs().childNodes());
+        if (iVisited.getRestArgNode() != null) {
+            arguments.add(new ConstNode(iVisited.getRestArgNode().getPosition(), '*' + iVisited.getRestArgNode().getName()));
         }
-        
-		if (iVisited.getBlockArgNode() != null) arguments.add(iVisited.getBlockArgNode());
+        if (iVisited.getPost() != null) arguments.addAll(iVisited.getPost().childNodes());
+		if (iVisited.getBlock() != null) arguments.add(iVisited.getBlock());
         
 		return arguments;
 	}
 	
 	private boolean hasNodeCommentsAtEnd(Node n) {
-		for (Iterator it = n.getComments().iterator(); it.hasNext(); ) {
-			Node comment = (Node) it.next();
-            
+	    for (Node comment: n.getComments()) {
 			if (getStartLine(comment) == getStartLine(n)) return true;
 		}
         
@@ -322,8 +318,8 @@ public class ReWriteVisitor implements NodeVisitor {
 	
 	public Instruction visitArgsNode(ArgsNode iVisited) {
 
-		for (Iterator it = collectAllArguments(iVisited).iterator(); it.hasNext(); ) {
-			Node n = (Node) it.next();
+		for (Iterator<Node> it = collectAllArguments(iVisited).iterator(); it.hasNext(); ) {
+			Node n = it.next();
             
 			if (n instanceof ArgumentNode) {
 				print(((ArgumentNode) n).getName());
@@ -532,12 +528,11 @@ public class ReWriteVisitor implements NodeVisitor {
         
 	private boolean printCommentsIn(Node iVisited) {
 		boolean hadComment = false;
-		for (Iterator it = iVisited.getComments().iterator(); it.hasNext(); ) {
-			CommentNode n = (CommentNode) it.next();
-            
+		for (CommentNode n: iVisited.getComments()) {
 			if(getStartLine(n) > getStartLine(iVisited) && getEndLine(n) < getEndLine(iVisited)) {
 				hadComment = true;
 				visitNode(n);
+                print(n.getContent());
 				printNewlineAndIndentation();
 			}
 		}
@@ -628,8 +623,8 @@ public class ReWriteVisitor implements NodeVisitor {
         
 		config.getPrintQuotesInString().set(false);
 		leaveCall();
-		for (Iterator it = iVisited.childNodes().iterator(); it.hasNext(); ) {
-            visitNode((Node) it.next());
+		for (Node child: iVisited.childNodes()) {
+            visitNode(child);
 		}
 		enterCall();
 		config.getPrintQuotesInString().revert();
@@ -645,8 +640,8 @@ public class ReWriteVisitor implements NodeVisitor {
         
         config.getPrintQuotesInString().set(false);
         leaveCall();
-        for (Iterator it = iVisited.childNodes().iterator(); it.hasNext(); ) {
-            visitNode((Node) it.next());
+        for (Node child: iVisited.childNodes()) {
+            visitNode(child);
         }        
         enterCall();
         config.getPrintQuotesInString().revert();
@@ -680,8 +675,8 @@ public class ReWriteVisitor implements NodeVisitor {
 	private boolean hasArguments(Node n) {
 		if (n instanceof ArgsNode) {
 			ArgsNode args = (ArgsNode) n;
-			return (args.getArgs() != null || args.getOptArgs() != null
-					|| args.getBlockArgNode() != null || args.getRestArg() > 0 );
+			return (args.getPre() != null || args.getOptArgs() != null
+					|| args.getBlock() != null || args.getRestArgNode() != null);
 		} else if (n instanceof ArrayNode && n.childNodes().isEmpty()) {
 			return false;
 		}
@@ -689,25 +684,25 @@ public class ReWriteVisitor implements NodeVisitor {
 	}
         
 	protected void printCommentsAtEnd(Node n) {
-		for (Iterator it = n.getComments().iterator(); it.hasNext(); ) {
-			Node comment = (Node) it.next();
-            
-			if(getStartLine(n) == getStartLine(comment)) {
+		for (CommentNode comment: n.getComments()) {
+			if (getStartLine(n) == getStartLine(comment)) {
 				print(' ');
 				visitNode(comment);
+                print(comment.getContent());
 			}
 		}
 	}
 	
 	private void printDefNode(Node parent, String name, Node args, StaticScope scope, Node bodyNode) {
 		print(name);
-		printCommentsAtEnd(parent);
 		config.getLocalVariables().addLocalVariable(scope);
 
 		if (hasArguments(args)) {
-			print(config.getFormatHelper().beforeMethodArguments());
-			visitNode(args);
+		    print(config.getFormatHelper().beforeMethodArguments());
+            visitNode(args);
 		}
+        printCommentsAtEnd(parent);
+        
 		visitNode(bodyNode);
 		config.getIndentor().outdent();
 		printNewlineAndIndentation();
@@ -849,7 +844,7 @@ public class ReWriteVisitor implements NodeVisitor {
 
 	private void printAsgnNode(AssignableNode n) {
 		print(((INameNode) n).getName());
-		if (n.getValueNode() == null) return;
+		if (n.getValueNode() == null || n.getValueNode().isInvisible()) return;
 		printAssignmentOperator();
 		visitNewlineInParentheses(n.getValueNode());
 	}
@@ -1009,7 +1004,7 @@ public class ReWriteVisitor implements NodeVisitor {
 		if (iVisited.getHeadNode() != null) {
 			factory.createMultipleAssignmentReWriteVisitor().visitAndPrintWithSeparator(iVisited.getHeadNode().childNodes().iterator());
         }
-		if (iVisited.getValueNode() == null) {
+		if (iVisited.getValueNode() == null || iVisited.getValueNode().isInvisible()) {
 			visitNode(iVisited.getArgsNode());
 			return null;
 		}
@@ -1026,6 +1021,27 @@ public class ReWriteVisitor implements NodeVisitor {
 		return null;
 	}
 
+	public Instruction visitMultipleAsgnNode(MultipleAsgn19Node iVisited) {
+		if (iVisited.getPre() != null) {
+			factory.createMultipleAssignmentReWriteVisitor().visitAndPrintWithSeparator(iVisited.getPre().childNodes().iterator());
+        }
+		if (iVisited.getValueNode() == null || iVisited.getValueNode().isInvisible()) {
+			visitNode(iVisited.getRest());
+			return null;
+		}
+		print(config.getFormatHelper().beforeAssignment());
+		print("=");
+		print(config.getFormatHelper().afterAssignment());
+		enterCall();
+		if (iVisited.getValueNode() instanceof ArrayNode) {
+			visitAndPrintWithSeparator(iVisited.getValueNode().childNodes().iterator());
+        } else {
+			visitNode(iVisited.getValueNode());
+        }
+		leaveCall();
+		return null;
+	}
+
 	public Instruction visitMatch2Node(Match2Node iVisited) {
 		visitNode(iVisited.getReceiverNode());
 		print(config.getFormatHelper().matchOperator());
@@ -1153,12 +1169,6 @@ public class ReWriteVisitor implements NodeVisitor {
 		return null;
 	}
 
-	public Instruction visitOptNNode(OptNNode iVisited) {
-		// this node is never used in the parser, only here:
-		// org.jruby/src/org/jruby/Main.java
-		return null;
-	}
-
 	public Instruction visitOrNode(OrNode iVisited) {
 		enterCall();
 		visitNode(iVisited.getFirstNode());
@@ -1179,7 +1189,13 @@ public class ReWriteVisitor implements NodeVisitor {
 		return null;
 	}
 
-	public Instruction visitRedoNode(RedoNode iVisited) {
+    public Instruction visitPreExeNode(PreExeNode iVisited) {
+        // this node contains nothing but an empty list, so we don't have to
+        // process anything
+        return null;
+    }
+
+    public Instruction visitRedoNode(RedoNode iVisited) {
 		print("redo");
 		return null;
 	}
@@ -1219,7 +1235,7 @@ public class ReWriteVisitor implements NodeVisitor {
 	}
 
 	public Instruction visitRescueBodyNode(RescueBodyNode iVisited) {
-		if (config.getLastPosition().getStartLine() == getEndLine(iVisited.getBodyNode())) {
+		if (!iVisited.getBodyNode().isInvisible() && config.getLastPosition().getStartLine() == getEndLine(iVisited.getBodyNode())) {
 			print(" rescue ");
 		} else {
 			print("rescue");
@@ -1269,12 +1285,12 @@ public class ReWriteVisitor implements NodeVisitor {
 		visitNode(iVisited.getBodyNode());
 		config.getIndentor().outdent();
 
-		if (iVisited.getRescueNode().getBodyNode() != null
+		if (!iVisited.getRescueNode().getBodyNode().isInvisible()
 				&& getStartLine(iVisited) != getEndLine(iVisited.getRescueNode().getBodyNode())) {
 			printNewlineAndIndentation();
         }
 
-		if (iVisited.getRescueNode().getBodyNode() == null) {
+		if (iVisited.getRescueNode().getBodyNode().isInvisible()) {
 			printNewlineAndIndentation();
 			print("rescue");
 			printExceptionNode(iVisited.getRescueNode());
@@ -1307,7 +1323,7 @@ public class ReWriteVisitor implements NodeVisitor {
 	public Instruction visitReturnNode(ReturnNode iVisited) {
 		print("return");
 		enterCall();
-		if (iVisited.getValueNode() != null) {
+		if (!iVisited.getValueNode().isInvisible()) {
 			print(' ');
 			visitNode(unwrapSingleArrayNode(iVisited.getValueNode()));
 		}
@@ -1659,4 +1675,9 @@ public class ReWriteVisitor implements NodeVisitor {
 
 		return null;
 	}
+
+    public Instruction visitRestArgNode(RestArgNode iVisited) {
+        print("*" + iVisited.getName());
+        return null;
+    }
 }
