@@ -39,12 +39,13 @@ import java.io.PrintStream;
 import java.io.PrintWriter;
 import java.io.StringWriter;
 
-import org.jruby.*;
+import org.jruby.NativeException;
+import org.jruby.Ruby;
+import org.jruby.RubyClass;
+import org.jruby.RubyException;
+import org.jruby.RubyString;
 import org.jruby.javasupport.util.RuntimeHelpers;
-import org.jruby.runtime.Block;
-import org.jruby.runtime.EventHook;
 import org.jruby.runtime.RubyEvent;
-import org.jruby.runtime.MethodIndex;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 
@@ -142,19 +143,30 @@ public class RaiseException extends JumpException {
 
         runtime.setStackTraces(runtime.getStackTraces() + 1);
 
-        StackTraceElement[] stackTrace = context.createBacktrace2(0, nativeException);
-        newException.setBacktraceFrames(stackTrace);
+        ThreadContext.RubyStackTraceElement[] stackTrace = newException.getBacktraceFrames();
+        if (stackTrace == null) {
+            stackTrace = context.createBacktrace2(0, nativeException);
+            newException.setBacktraceFrames(stackTrace);
+        }
 
         // JRUBY-2673: if wrapping a NativeException, use the actual Java exception's trace as our Java trace
         if (newException instanceof NativeException) {
             setStackTrace(((NativeException)newException).getCause().getStackTrace());
         } else {
-            setStackTrace(stackTrace);
+            setStackTrace(javaTraceFromRubyTrace(stackTrace));
         }
 
         runtime.setStackTraces(runtime.getStackTraces() - 1);
     }
 
+    private StackTraceElement[] javaTraceFromRubyTrace(ThreadContext.RubyStackTraceElement[] trace) {
+        StackTraceElement[] newTrace = new StackTraceElement[trace.length];
+        for (int i = 0; i < newTrace.length; i++) {
+            newTrace[i] = trace[i].getElement();
+        }
+        return newTrace;
+    }
+
     public void printStackTrace() {
         printStackTrace(System.err);
     }
