@@ -47,7 +47,7 @@ import org.jruby.util.ByteList;
  * A abstract memory object that defines operations common to both pointers and
  * memory buffers
  */
-@JRubyClass(name=AbstractMemory.ABSTRACT_MEMORY_RUBY_CLASS, parent="Object")
+@JRubyClass(name="FFI::" + AbstractMemory.ABSTRACT_MEMORY_RUBY_CLASS, parent="Object")
 abstract public class AbstractMemory extends RubyObject {
     public final static String ABSTRACT_MEMORY_RUBY_CLASS = "AbstractMemory";
 
@@ -58,8 +58,7 @@ abstract public class AbstractMemory extends RubyObject {
     /** The Memory I/O object */
     protected final MemoryIO io;
     
-    public static RubyClass createAbstractMemoryClass(Ruby runtime) {
-        RubyModule module = FFIProvider.getModule(runtime);
+    public static RubyClass createAbstractMemoryClass(Ruby runtime, RubyModule module) {
         RubyClass result = module.defineClassUnder(ABSTRACT_MEMORY_RUBY_CLASS,
                 runtime.getObject(),
                 ObjectAllocator.NOT_ALLOCATABLE_ALLOCATOR);
@@ -105,6 +104,15 @@ abstract public class AbstractMemory extends RubyObject {
         return this.offset;
     }
 
+    /**
+     * Gets the size of the memory area.
+     *
+     * @return The size of the memory area.
+     */
+    protected final long getSize() {
+        return this.size;
+    }
+
     /**
      * Calculates a hash code for the pointer.
      *
@@ -639,16 +647,18 @@ abstract public class AbstractMemory extends RubyObject {
         getMemoryIO().put(getOffset(offset), array, 0, array.length);
         return this;
     }
-    @JRubyMethod(name = "get_string")
+    @JRubyMethod(name = "get_string", required = 1)
     public IRubyObject get_string(ThreadContext context, IRubyObject offArg) {
         long off = getOffset(offArg);
         int len = (int) getMemoryIO().indexOf(off, (byte) 0);
         ByteList bl = new ByteList(len);
         getMemoryIO().get(off, bl.unsafeBytes(), bl.begin(), len);
         bl.length(len);
-        return context.getRuntime().newString(bl);
+        RubyString s = context.getRuntime().newString(bl);
+        s.setTaint(true);
+        return s;
     }
-    @JRubyMethod(name = "get_string")
+    @JRubyMethod(name = "get_string", required = 2)
     public IRubyObject get_string(ThreadContext context, IRubyObject offArg, IRubyObject lenArg) {
         long off = getOffset(offArg);
         int maxlen = Util.int32Value(lenArg);
@@ -659,30 +669,65 @@ abstract public class AbstractMemory extends RubyObject {
         ByteList bl = new ByteList(len);
         getMemoryIO().get(off, bl.unsafeBytes(), bl.begin(), len);
         bl.length(len);
-        return context.getRuntime().newString(bl);
+        RubyString s = context.getRuntime().newString(bl);
+        s.setTaint(true);
+        return s;
     }
-    @JRubyMethod(name = "put_string", required = 2, optional = 1)
-    public IRubyObject put_string(ThreadContext context, IRubyObject[] args) {
+    @JRubyMethod(name = "put_string")
+    public IRubyObject put_string(ThreadContext context, IRubyObject offArg, IRubyObject strArg) {
+        long off = getOffset(offArg);
+        ByteList bl = strArg.convertToString().getByteList();
+
+        getMemoryIO().put(off, bl.unsafeBytes(), bl.begin(), bl.length());
+        getMemoryIO().putByte(off + bl.length(), (byte) 0);
+        return this;
+    }
+    @JRubyMethod(name = "get_bytes")
+    public IRubyObject get_bytes(ThreadContext context, IRubyObject offArg, IRubyObject lenArg) {
+        long off = getOffset(offArg);
+        int len = Util.int32Value(lenArg);
+        ByteList bl = new ByteList(len);
+        checkBounds(context, off, len);
+        getMemoryIO().get(off, bl.unsafeBytes(), bl.begin(), len);
+        bl.length(len);
+        RubyString s = context.getRuntime().newString(bl);
+        s.setTaint(true);
+        return s;
+    }
+    @JRubyMethod(name = "put_bytes", required = 2, optional = 2)
+    public IRubyObject put_bytes(ThreadContext context, IRubyObject[] args) {
         long off = getOffset(args[0]);
         ByteList bl = args[1].convertToString().getByteList();
-        int len = bl.length();
-        boolean nulTerminate = true;
-        if (args.length > 2) {
-            len = Math.min(Util.int32Value(args[2]) - 1, len);
-            nulTerminate = false;
+        int idx = args.length > 2 ? Util.int32Value(args[2]) : 0;
+        if (idx < 0 || idx > bl.length()) {
+            throw context.getRuntime().newRangeError("Invalid string index");
         }
-        getMemoryIO().put(off, bl.unsafeBytes(), bl.begin(), len);
-        if (nulTerminate) {
-            getMemoryIO().putByte(off + bl.length(), (byte) 0);
+        int len = args.length > 3 ? Util.int32Value(args[3]) : (bl.length() - idx);
+        if (len < 0 || len > (bl.length() - idx)) {
+            throw context.getRuntime().newRangeError("Invalid length");
         }
+        checkBounds(context, off, len);
+        getMemoryIO().put(off, bl.unsafeBytes(), bl.begin() + idx, len);
         return this;
     }
     @JRubyMethod(name = "get_pointer", required = 1)
     public IRubyObject get_pointer(ThreadContext context, IRubyObject offset) {
-        return getMemoryPointer(context.getRuntime(), Util.int64Value(offset));
+        return getPointer(context.getRuntime(), Util.int64Value(offset));
     }
-    
-    abstract protected AbstractMemoryPointer getMemoryPointer(Ruby runtime, long offset);
-    
-    
+    @JRubyMethod(name = "__get_buffer", required = 2)
+    public IRubyObject get_buffer(ThreadContext context, IRubyObject off, IRubyObject len_) {
+        int len = Util.int32Value(len_);
+        ByteList bl = new ByteList(len);
+        getMemoryIO().get(getOffset(off), bl.unsafeBytes(), bl.begin(), len);
+        bl.length(len);
+        return context.getRuntime().newString(bl);
+    }
+    @JRubyMethod(name = "__put_buffer", required = 3)
+    public IRubyObject put_buffer(ThreadContext context, IRubyObject off, IRubyObject str, IRubyObject len_) {
+        ByteList bl = str.convertToString().getByteList();
+        int len = Math.min(bl.length(), Util.int32Value(len_));
+        getMemoryIO().put(getOffset(off), bl.unsafeBytes(), bl.begin(), len);
+        return context.getRuntime().newFixnum(len);
+    }
+    abstract protected Pointer getPointer(Ruby runtime, long offset);
 }
