@@ -73,10 +73,9 @@ public final class StructLayoutBuilder extends RubyObject {
         }
         private static final ObjectAllocator INSTANCE = new Allocator();
     }
-    public static RubyClass createStructLayoutBuilderClass(Ruby runtime) {
-        RubyModule parent = FFIProvider.getModule(runtime);
+    public static RubyClass createStructLayoutBuilderClass(Ruby runtime, RubyModule module) {
         RubyClass result = runtime.defineClassUnder(CLASS_NAME, runtime.getObject(),
-                Allocator.INSTANCE, parent);
+                Allocator.INSTANCE, module);
         result.defineAnnotatedMethods(StructLayoutBuilder.class);
         result.defineAnnotatedConstants(StructLayoutBuilder.class);
 
@@ -336,7 +335,7 @@ public final class StructLayoutBuilder extends RubyObject {
         }
 
         public IRubyObject get(Ruby runtime, IRubyObject ptr) {
-            return ((AbstractMemory) ptr).getMemoryPointer(runtime, offset);
+            return ((AbstractMemory) ptr).getPointer(runtime, offset);
         }
         static StructLayout.Member create(long offset) { return new PointerMember(offset); }
     }
