@@ -28,7 +28,6 @@
 
 package org.jruby.ext.ffi.jna;
 
-import com.sun.jna.CallbackProxy;
 import org.jruby.ext.ffi.*;
 import com.sun.jna.Function;
 import com.sun.jna.Memory;
@@ -37,9 +36,7 @@ import com.sun.jna.NativeLibrary;
 import com.sun.jna.Pointer;
 import java.nio.ByteBuffer;
 import org.jruby.Ruby;
-import org.jruby.RubyProc;
 import org.jruby.RubyString;
-import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.ByteList;
 
@@ -65,12 +62,6 @@ public final class JNAProvider extends FFIProvider {
 
         return new JNAInvoker(runtime, function, functionInvoker, marshallers);
     }
-
-    @Override
-    public Callback createCallback(Ruby runtime, NativeType returnType,
-            NativeType[] parameterTypes) {
-        return new Callback(runtime, returnType, parameterTypes);
-    }
     
     public int getLastError() {
         return Native.getLastError();
@@ -86,23 +77,28 @@ public final class JNAProvider extends FFIProvider {
      * @param returnType The return type of the native function.
      * @return A new <tt>FunctionInvoker</tt> to invoke the native function.
      */
-    private FunctionInvoker getFunctionInvoker(NativeType returnType) {
+    static FunctionInvoker getFunctionInvoker(NativeType returnType) {
         switch (returnType) {
             case VOID:
                 return VoidInvoker.INSTANCE;
             case POINTER:
                 return PointerInvoker.INSTANCE;
             case INT8:
+                return Signed8Invoker.INSTANCE;
             case INT16:
+                return Signed16Invoker.INSTANCE;
             case INT32:
                 return Signed32Invoker.INSTANCE;
             case UINT8:
+                return Unsigned8Invoker.INSTANCE;
             case UINT16:
+                return Unsigned16Invoker.INSTANCE;
             case UINT32:
                 return Unsigned32Invoker.INSTANCE;
             case INT64:
-            case UINT64:
                 return Signed64Invoker.INSTANCE;
+            case UINT64:
+                return Unsigned64Invoker.INSTANCE;
             case LONG:
                 return Platform.getPlatform().addressSize() == 32
                         ? Signed32Invoker.INSTANCE
@@ -128,11 +124,11 @@ public final class JNAProvider extends FFIProvider {
      * @param type The native type to convert to.
      * @return A new <tt>Marshaller</tt>
      */
-    private static final Marshaller getMarshaller(NativeParam type, int convention) {
+    static final Marshaller getMarshaller(NativeParam type, int convention) {
         if (type instanceof NativeType) {
             return getMarshaller((NativeType) type);
-        } else if (type instanceof org.jruby.ext.ffi.Callback) {
-            return new CallbackMarshaller((org.jruby.ext.ffi.Callback) type, convention);
+        } else if (type instanceof org.jruby.ext.ffi.CallbackInfo) {
+            return new CallbackMarshaller((org.jruby.ext.ffi.CallbackInfo) type, convention);
         } else {
             return null;
         }        
@@ -143,7 +139,7 @@ public final class JNAProvider extends FFIProvider {
      * @param type The native type to convert to.
      * @return A new <tt>Marshaller</tt>
      */
-    private static final Marshaller getMarshaller(NativeType type) {
+    static final Marshaller getMarshaller(NativeType type) {
         switch (type) {
             case INT8:
                 return Signed8Marshaller.INSTANCE;
@@ -198,7 +194,51 @@ public final class JNAProvider extends FFIProvider {
         }
         public static final FunctionInvoker INSTANCE = new VoidInvoker();
     }
-    
+    /**
+     * Invokes the native function with n signed 8 bit integer return value.
+     * Returns a Fixnum to ruby.
+     */
+    private static final class Signed8Invoker implements FunctionInvoker {
+        public final IRubyObject invoke(Ruby runtime, Function function, Object[] args) {
+            int value = function.invokeInt(args) & 0xff;
+            return runtime.newFixnum(value < 0x80 ? value : -0x80 + (value - 0x80));
+        }
+        public static final FunctionInvoker INSTANCE = new Signed8Invoker();
+    }
+
+    /**
+     * Invokes the native function with an unsigned 8 bit integer return value.
+     * Returns a Fixnum to ruby.
+     */
+    private static final class Unsigned8Invoker implements FunctionInvoker {
+        public final IRubyObject invoke(Ruby runtime, Function function, Object[] args) {
+            return Util.newUnsigned8(runtime, function.invokeInt(args));
+        }
+        public static final FunctionInvoker INSTANCE = new Unsigned8Invoker();
+    }
+
+    /**
+     * Invokes the native function with n signed 8 bit integer return value.
+     * Returns a Fixnum to ruby.
+     */
+    private static final class Signed16Invoker implements FunctionInvoker {
+        public final IRubyObject invoke(Ruby runtime, Function function, Object[] args) {
+            int value = function.invokeInt(args) & 0xffff;
+            return runtime.newFixnum(value < 0x8000 ? value : -0x8000 + (value - 0x8000));
+        }
+        public static final FunctionInvoker INSTANCE = new Signed16Invoker();
+    }
+
+    /**
+     * Invokes the native function with an unsigned 32 bit integer return value.
+     * Returns a Fixnum to ruby.
+     */
+    private static final class Unsigned16Invoker implements FunctionInvoker {
+        public final IRubyObject invoke(Ruby runtime, Function function, Object[] args) {
+            return Util.newUnsigned16(runtime, function.invokeInt(args));
+        }
+        public static final FunctionInvoker INSTANCE = new Unsigned16Invoker();
+    }
     /**
      * Invokes the native function with a 32 bit integer return value.
      * Returns a Fixnum to ruby.
@@ -231,6 +271,17 @@ public final class JNAProvider extends FFIProvider {
         }
         public static final FunctionInvoker INSTANCE = new Signed64Invoker();
     }
+
+    /**
+     * Invokes the native function with a 64 bit unsigned integer return value.
+     * Returns a ruby Fixnum or Bignum.
+     */
+    private static final class Unsigned64Invoker implements FunctionInvoker {
+        public final IRubyObject invoke(Ruby runtime, Function function, Object[] args) {
+            return Util.newUnsigned64(runtime, function.invokeLong(args));
+        }
+        public static final FunctionInvoker INSTANCE = new Unsigned64Invoker();
+    }
     
     /**
      * Invokes the native function with a 32 bit float return value.
@@ -260,7 +311,7 @@ public final class JNAProvider extends FFIProvider {
      */
     private static final class PointerInvoker implements FunctionInvoker {
         public final IRubyObject invoke(Ruby runtime, Function function, Object[] args) {
-            return new JNAMemoryPointer(runtime, function.invokePointer(args));
+            return new JNABasePointer(runtime, function.invokePointer(args));
         }
         public static final FunctionInvoker INSTANCE = new PointerInvoker();
     }
@@ -283,7 +334,9 @@ public final class JNAProvider extends FFIProvider {
             bl.length(len);
             address.read(0, bl.unsafeBytes(), bl.begin(), len);
             
-            return RubyString.newString(runtime, bl);
+            RubyString s =  RubyString.newString(runtime, bl);
+            s.setTaint(true);
+            return s;
         }
         public static final FunctionInvoker INSTANCE = new StringInvoker();
     }
@@ -291,7 +344,7 @@ public final class JNAProvider extends FFIProvider {
     /**
      * Converts a ruby Fixnum into an 8 bit native integer.
      */
-    private static final class Signed8Marshaller implements Marshaller {
+    static final class Signed8Marshaller implements Marshaller {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
             return Byte.valueOf(Util.int8Value(parameter));
         }
@@ -301,7 +354,7 @@ public final class JNAProvider extends FFIProvider {
     /**
      * Converts a ruby Fixnum into an 8 bit native unsigned integer.
      */
-    private static final class Unsigned8Marshaller implements Marshaller {
+    static final class Unsigned8Marshaller implements Marshaller {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
             return Byte.valueOf((byte) Util.uint8Value(parameter));
         }
@@ -311,7 +364,7 @@ public final class JNAProvider extends FFIProvider {
     /**
      * Converts a ruby Fixnum into a 16 bit native signed integer.
      */
-    private static final class Signed16Marshaller implements Marshaller {
+    static final class Signed16Marshaller implements Marshaller {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
             return Short.valueOf(Util.int16Value(parameter));
         }
@@ -321,7 +374,7 @@ public final class JNAProvider extends FFIProvider {
     /**
      * Converts a ruby Fixnum into a 16 bit native unsigned integer.
      */
-    private static final class Unsigned16Marshaller implements Marshaller {
+    static final class Unsigned16Marshaller implements Marshaller {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
             return Short.valueOf((short) Util.uint16Value(parameter));
         }
@@ -331,7 +384,7 @@ public final class JNAProvider extends FFIProvider {
     /**
      * Converts a ruby Fixnum into a 32 bit native signed integer.
      */
-    private static final class Signed32Marshaller implements Marshaller {
+    static final class Signed32Marshaller implements Marshaller {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
             return Integer.valueOf(Util.int32Value(parameter));
         }
@@ -341,7 +394,7 @@ public final class JNAProvider extends FFIProvider {
     /**
      * Converts a ruby Fixnum into a 32 bit native unsigned integer.
      */
-    private static final class Unsigned32Marshaller implements Marshaller {
+    static final class Unsigned32Marshaller implements Marshaller {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
             return Integer.valueOf((int) Util.uint32Value(parameter));
         }
@@ -351,7 +404,7 @@ public final class JNAProvider extends FFIProvider {
     /**
      * Converts a ruby Fixnum into a 64 bit native signed integer.
      */
-    private static final class Signed64Marshaller implements Marshaller {
+    static final class Signed64Marshaller implements Marshaller {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
             return Long.valueOf(Util.int64Value(parameter));
         }
@@ -361,7 +414,7 @@ public final class JNAProvider extends FFIProvider {
     /**
      * Converts a ruby Fixnum into a 64 bit native unsigned integer.
      */
-    private static final class Unsigned64Marshaller implements Marshaller {
+    static final class Unsigned64Marshaller implements Marshaller {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
             return Long.valueOf(Util.uint64Value(parameter));
         }
@@ -371,7 +424,7 @@ public final class JNAProvider extends FFIProvider {
     /**
      * Converts a ruby Float into a 32 bit native float.
      */
-    private static final class Float32Marshaller implements Marshaller {
+    static final class Float32Marshaller implements Marshaller {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
             return Float.valueOf(Util.floatValue(parameter));
         }
@@ -381,7 +434,7 @@ public final class JNAProvider extends FFIProvider {
     /**
      * Converts a ruby Float into a 64 bit native float.
      */
-    private static final class Float64Marshaller implements Marshaller {
+    static final class Float64Marshaller implements Marshaller {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
             return Double.valueOf(Util.doubleValue(parameter));
         }
@@ -391,7 +444,7 @@ public final class JNAProvider extends FFIProvider {
     /**
      * Converts a ruby MemoryPointer into a native pointer.
      */
-    private static final class PointerMarshaller implements Marshaller {
+    static final class PointerMarshaller implements Marshaller {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
             if (parameter instanceof JNAMemory) {
                 return (((JNAMemory) parameter).getNativeMemory());
@@ -413,8 +466,13 @@ public final class JNAProvider extends FFIProvider {
                     }
                 });
                 return memory;
+            } else if (parameter.respondsTo("to_ptr")) {
+                IRubyObject ptr = parameter.callMethod(invocation.getThreadContext(), "to_ptr");
+                if (ptr instanceof JNAMemory) {
+                    return (((JNAMemory) ptr).getNativeMemory());
+                }
             }
-            return Util.convertParameter(parameter, Pointer.class);
+            throw invocation.getThreadContext().getRuntime().newArgumentError("Invalid Pointer argument");
         }
         public static final Marshaller INSTANCE = new PointerMarshaller();
     }
@@ -430,6 +488,9 @@ public final class JNAProvider extends FFIProvider {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
             // Ruby strings are UTF-8, so should be able to just copy directly
             RubyString s = parameter.asString();
+            if (invocation.getThreadContext().getRuntime().getSafeLevel() > 0 && s.isTaint()) {
+                throw invocation.getThreadContext().getRuntime().newSecurityError("Unsafe string parameter");
+            }
             ByteList bl = s.getByteList();
             final Memory memory = new Memory(bl.length() + 1);
             memory.write(0, bl.unsafeBytes(), bl.begin(), bl.length());
@@ -480,9 +541,13 @@ public final class JNAProvider extends FFIProvider {
             } else if (parameter instanceof RubyString) {
                 ByteList bl = ((RubyString) parameter).getByteList();
                 return ByteBuffer.wrap(bl.unsafeBytes(), bl.begin(), bl.realSize);
-            } else { 
-                return Util.convertParameter(parameter, Object.class);
+            } else if (parameter.respondsTo("to_ptr")) {
+                IRubyObject ptr = parameter.callMethod(invocation.getThreadContext(), "to_ptr");
+                if (ptr instanceof JNAMemory) {
+                    return (((JNAMemory) ptr).getNativeMemory());
+                }
             }
+            throw invocation.getThreadContext().getRuntime().newArgumentError("Invalid Buffer argument");
         }
         public static final Marshaller INSTANCE = new BufferMarshaller();
     }
