@@ -32,6 +32,8 @@ import org.jruby.Ruby;
 import org.jruby.RubyProc;
 import org.jruby.RubyThread;
 import org.jruby.RubyThreadGroup;
+import org.jruby.exceptions.JumpException;
+import org.jruby.exceptions.MainExitException;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.exceptions.ThreadKill;
 import org.jruby.runtime.Block;
@@ -39,50 +41,64 @@ import org.jruby.runtime.Frame;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 
+@Deprecated
 public class RubyNativeThread extends Thread {
     private Ruby runtime;
     private Frame currentFrame;
     private RubyProc proc;
     private IRubyObject[] arguments;
     private RubyThread rubyThread;
+    private boolean setContextCC;
     
-    public RubyNativeThread(RubyThread rubyThread, IRubyObject[] args, Block currentBlock) {
-        super(rubyThread.getRuntime().getThreadService().getRubyThreadGroup(), "Ruby Thread" + rubyThread.hash());
-        setDaemon(true);
-        this.rubyThread = rubyThread;
-        this.runtime = rubyThread.getRuntime();
-        ThreadContext tc = runtime.getCurrentContext();
-        
-        proc = runtime.newProc(Block.Type.PROC, currentBlock);
-        currentFrame = tc.getCurrentFrame();
-        this.arguments = args;
+    public RubyNativeThread(RubyThread rubyThread, IRubyObject[] args, Block currentBlock, boolean setContextCC) {
+        throw new RuntimeException("RubyNativeThread is deprecated; do not use it");
     }
     
     public RubyThread getRubyThread() {
         return rubyThread;
     }
     
+    private static boolean warnedAboutTC = false;
+    
     public void run() {
-        runtime.getThreadService().registerNewThread(rubyThread);
-        ThreadContext context = runtime.getCurrentContext();
+        ThreadContext context = runtime.getThreadService().registerNewThread(rubyThread);
+        
+        // set thread context JRuby classloader here, for Ruby-owned thread
+        try {
+            Thread.currentThread().setContextClassLoader(runtime.getJRubyClassLoader());
+        } catch (SecurityException se) {
+            // can't set TC classloader
+            if (!warnedAboutTC && runtime.getInstanceConfig().isVerbose()) {
+                System.err.println("WARNING: Security restrictions disallowed setting context classloader for Ruby threads.");
+            }
+        }
         
         context.preRunThread(currentFrame);
-        
+
         // Call the thread's code
         try {
-            IRubyObject result = proc.call(arguments);
+            IRubyObject result = proc.call(context, arguments);
             rubyThread.cleanTerminate(result);
         } catch (ThreadKill tk) {
             // notify any killer waiting on our thread that we're going bye-bye
             synchronized (rubyThread.killLock) {
                 rubyThread.killLock.notifyAll();
             }
+        } catch (JumpException.ReturnJump rj) {
+            rubyThread.exceptionRaised(runtime.newThreadError("return can't jump across threads"));
         } catch (RaiseException e) {
             rubyThread.exceptionRaised(e);
+        } catch (MainExitException mee) {
+            // Someone called exit!, so we need to kill the main thread
+            runtime.getThreadService().getMainThread().kill();
         } finally {
             runtime.getThreadService().setCritical(false);
             runtime.getThreadService().unregisterThread(rubyThread);
-            ((RubyThreadGroup)rubyThread.group()).remove(rubyThread);
+            
+            // synchronize on the RubyThread object for threadgroup updates
+            synchronized (rubyThread) {
+                ((RubyThreadGroup)rubyThread.group()).remove(rubyThread);
+            }
         }
     }
 }
