@@ -11,13 +11,8 @@
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
- * Copyright (C) 2002 Benoit Cerrina <b.cerrina@wanadoo.fr>
- * Copyright (C) 2002-2004 Anders Bengtsson <ndrsbngtssn@yahoo.se>
- * Copyright (C) 2002-2004 Jan Arne Petersen <jpetersen@uni-bonn.de>
- * Copyright (C) 2004-2005 Thomas E Enebo <enebo@acm.org>
- * Copyright (C) 2004 Stefan Matthias Aust <sma@3plus4.de>
- * Copyright (C) 2006 Miguel Covarrubias <mlcovarrubias@gmail.com>
- * 
+ * Copyright (C) 2004-2008 Thomas E Enebo <enebo@acm.org>
+ *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
  * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
@@ -52,10 +47,14 @@ import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
 
 /**
+ * This is the mixed-mode method type.  It will call out to JIT compiler to see if the compiler
+ * wants to JIT or not.  If the JIT compiler does JIT this it will return the new method
+ * to be executed here instead of executing the interpreted version of this method.  The next
+ * invocation of the method will end up causing the runtime to load and execute the newly JIT'd
+ * method.
  *
  */
-public final class DefaultMethod extends DynamicMethod implements JumpTarget, MethodArgs {
-    
+public class DefaultMethod extends DynamicMethod implements JumpTarget, MethodArgs {
     private StaticScope staticScope;
     private Node body;
     private ArgsNode argsNode;
@@ -68,8 +67,8 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
     private CallConfiguration jitCallConfig;
     private ISourcePosition position;
     private boolean noArgHack;
-
-    public DefaultMethod(RubyModule implementationClass, StaticScope staticScope, Node body, 
+    
+    public DefaultMethod(RubyModule implementationClass, StaticScope staticScope, Node body,
             ArgsNode argsNode, Visibility visibility, ISourcePosition position) {
         super(implementationClass, visibility, CallConfiguration.FRAME_AND_SCOPE);
         this.body = body;
@@ -80,69 +79,61 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
         this.restArg = argsNode.getRestArg();
         this.hasOptArgs = argsNode.getOptArgs() != null;
         this.position = position;
-        
-        noArgHack = argsNode.getBlockArgNode() == null && requiredArgsCount == 0 && maxArgsCount == 0;
-		
+
+        noArgHack = argsNode.getBlock() == null && requiredArgsCount == 0 && maxArgsCount == 0;
+
         assert argsNode != null;
     }
-    
+
     public int getCallCount() {
         return callCount;
     }
-    
+
     public int incrementCallCount() {
         return ++callCount;
     }
-    
+
     public void setCallCount(int callCount) {
         this.callCount = callCount;
     }
-    
-    public Script getJITCompilerScript() {
-        return jitCompiledScript;
-    }
-    
-    public void setJITCompiledScript(Script jitCompiledScript) {
-        this.jitCompiledScript = jitCompiledScript;
-    }
-    
-    public CallConfiguration getJITCallConfig() {
-        return jitCallConfig;
-    }
-    
-    public void setJITCallConfig(CallConfiguration jitCallConfig) {
-        this.jitCallConfig = jitCallConfig;
-    }
-    
+
     public Node getBodyNode() {
         return body;
     }
-    
+
     public ArgsNode getArgsNode() {
         return argsNode;
     }
-    
+
     public StaticScope getStaticScope() {
         return staticScope;
     }
 
+    public void setJITCompiledScript(Script jitCompiledScript) {
+        this.jitCompiledScript = jitCompiledScript;
+    }
+
+    public void setJITCallConfig(CallConfiguration jitCallConfig) {
+        this.jitCallConfig = jitCallConfig;
+    }
+
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args, Block block) {
         assert args != null;
-        
+
         Ruby runtime = context.getRuntime();
 
         if (callCount >= 0) {
             runtime.getJITCompiler().tryJIT(this, context, name);
         }
-        
+
         if (jitCompiledScript != null && !runtime.hasEventHooks()) {
             return retryJITCall(context, runtime, self, name, args, block);
         } else {
             return interpretedCall(context, runtime, self, clazz, name, args, block);
         }
     }
-    
+
     private IRubyObject retryJITCall(ThreadContext context, Ruby runtime, IRubyObject self, String name, IRubyObject[] args, Block block) {
         try {
             jitPre(context, self, name, block, args.length);
@@ -175,11 +166,11 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args) {
         Ruby runtime = context.getRuntime();
-        
+
         if (jitCompiledScript != null && !runtime.hasEventHooks()) {
             try {
                 jitPre(context, self, name, Block.NULL_BLOCK, args.length);
-                
+
                 return jitCompiledScript.__file__(context, self, args, Block.NULL_BLOCK);
             } catch (JumpException.ReturnJump rj) {
                 return handleReturn(context, rj);
@@ -196,7 +187,7 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name) {
         Ruby runtime = context.getRuntime();
-        
+
         if (jitCompiledScript != null && !runtime.hasEventHooks()) {
             try {
                 jitPre(context, self, name, Block.NULL_BLOCK, 0);
@@ -217,7 +208,7 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, Block block) {
         Ruby runtime = context.getRuntime();
-        
+
         if (jitCompiledScript != null && !runtime.hasEventHooks()) {
             try {
                 jitPre(context, self, name, block, 0);
@@ -238,7 +229,7 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject arg0) {
         Ruby runtime = context.getRuntime();
-        
+
         if (jitCompiledScript != null && !runtime.hasEventHooks()) {
             try {
                 jitPre(context, self, name, Block.NULL_BLOCK, 1);
@@ -259,7 +250,7 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject arg0, Block block) {
         Ruby runtime = context.getRuntime();
-        
+
         if (jitCompiledScript != null && !runtime.hasEventHooks()) {
             try {
                 jitPre(context, self, name, block, 1);
@@ -280,7 +271,7 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject arg0, IRubyObject arg1) {
         Ruby runtime = context.getRuntime();
-        
+
         if (jitCompiledScript != null && !runtime.hasEventHooks()) {
             try {
                 jitPre(context, self, name, Block.NULL_BLOCK, 2);
@@ -301,7 +292,7 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject arg0, IRubyObject arg1, Block block) {
         Ruby runtime = context.getRuntime();
-        
+
         if (jitCompiledScript != null && !runtime.hasEventHooks()) {
             try {
                 jitPre(context, self, name, block, 2);
@@ -322,7 +313,7 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2) {
         Ruby runtime = context.getRuntime();
-        
+
         if (jitCompiledScript != null && !runtime.hasEventHooks()) {
             try {
                 jitPre(context, self, name, Block.NULL_BLOCK, 3);
@@ -343,7 +334,7 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, Block block) {
         Ruby runtime = context.getRuntime();
-        
+
         if (jitCompiledScript != null && !runtime.hasEventHooks()) {
             try {
                 jitPre(context, self, name, block, 3);
@@ -385,7 +376,7 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
 
         prepareArguments(context, runtime, self, args);
 
-        if (argsNode.getBlockArgNode() != null) {
+        if (argsNode.getBlock() != null) {
             processBlockArg(context, block);
         }
     }
@@ -416,16 +407,16 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
         if (runtime.hasEventHooks()) {
             traceCall(context, runtime, name);
         }
-        checkArgCount(runtime, args.length);    
+        checkArgCount(runtime, args.length);
         if (!noArgHack) interpretArgs(runtime, args, context, self, block);
     }
-    
+
     private void processBlockArg(ThreadContext context, Block block) {
         context.getCurrentScope().setValue(
-                argsNode.getBlockArgNode().getCount(),
+                argsNode.getBlock().getCount(),
                 RuntimeHelpers.processBlockArgument(context.getRuntime(), block), 0);
     }
-    
+
     private void checkArgCount(Ruby runtime, int argsLength) {
         Arity.checkArgumentCount(runtime, argsLength, requiredArgsCount, maxArgsCount);
     }
@@ -441,7 +432,7 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
             prepareOptOrRestArgs(context, runtime, self, args);
         }
     }
-    
+
     private void prepareOptOrRestArgs(ThreadContext context, Ruby runtime, IRubyObject self, IRubyObject[] args) {
         int givenArgsCount = interpretOptArgs(context, runtime, self, args);
         interpretRestArg(context, runtime, args, givenArgsCount);
@@ -459,7 +450,7 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
             context.getCurrentScope().setValue(restArg, array, 0);
         }
     }
-    
+
     public ISourcePosition getPosition() {
         return position;
     }
@@ -467,7 +458,7 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
     private void traceReturn(ThreadContext context, Ruby runtime, String name) {
         runtime.callEventHooks(context, RubyEvent.RETURN, context.getFile(), context.getLine(), name, getImplementationClass());
     }
-    
+
     private void traceCall(ThreadContext context, Ruby runtime, String name) {
         runtime.callEventHooks(context, RubyEvent.CALL, position.getFile(), position.getStartLine(), name, getImplementationClass());
     }
@@ -476,8 +467,10 @@ public final class DefaultMethod extends DynamicMethod implements JumpTarget, Me
     public Arity getArity() {
         return argsNode.getArity();
     }
-    
+
     public DynamicMethod dup() {
         return new DefaultMethod(getImplementationClass(), staticScope, body, argsNode, getVisibility(), position);
     }
+
+
 }
