@@ -81,11 +81,14 @@ import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.runtime.callback.Callback;
 import org.jruby.util.ByteList;
+import org.jruby.util.CodegenUtils;
 import org.jruby.util.IdUtil;
 
 
 @JRubyClass(name="Java::JavaClass", parent="Java::JavaObject")
 public class JavaClass extends JavaObject {
+    public static final String METHOD_MANGLE = "__method";
+
     /**
      * Assigned names only override based priority of an assigned type, the type must be less than
      * or equal to the assigned type. For example, field name (FIELD) in a subclass will override
@@ -712,6 +715,9 @@ public class JavaClass extends JavaObject {
             Method method = methods[i];
             String name = method.getName();
             if (Modifier.isStatic(method.getModifiers())) {
+                // Install direct java methods with mangled name so 'send' can call them directly.
+                installStaticMethods(staticCallbacks, javaClass, method, name + METHOD_MANGLE);
+                
                 AssignedName assignedName = staticNames.get(name);
                 if (assignedName == null) {
                     staticNames.put(name, new AssignedName(name, Priority.METHOD));
@@ -723,13 +729,11 @@ public class JavaClass extends JavaObject {
                         staticNames.put(name, new AssignedName(name, Priority.METHOD));
                     }
                 }
-                StaticMethodInvokerInstaller invoker = (StaticMethodInvokerInstaller) staticCallbacks.get(name);
-                if (invoker == null) {
-                    invoker = new StaticMethodInvokerInstaller(name);
-                    staticCallbacks.put(name, invoker);
-                }
-                invoker.addMethod(method, javaClass);
+                installStaticMethods(staticCallbacks, javaClass, method, name);
             } else {
+                // Install direct java methods with mangled name so 'send' can call them directly.
+                installInstanceMethods(instanceCallbacks, javaClass, method, name + METHOD_MANGLE);
+
                 AssignedName assignedName = instanceNames.get(name);
                 if (assignedName == null) {
                     instanceNames.put(name, new AssignedName(name, Priority.METHOD));
@@ -741,15 +745,28 @@ public class JavaClass extends JavaObject {
                         instanceNames.put(name, new AssignedName(name, Priority.METHOD));
                     }
                 }
-                InstanceMethodInvokerInstaller invoker = (InstanceMethodInvokerInstaller) instanceCallbacks.get(name);
-                if (invoker == null) {
-                    invoker = new InstanceMethodInvokerInstaller(name);
-                    instanceCallbacks.put(name, invoker);
-                }
-                invoker.addMethod(method, javaClass);
+                installInstanceMethods(instanceCallbacks, javaClass, method, name);
             }
         }
     }
+
+    private void installInstanceMethods(Map<String, NamedInstaller> methodCallbacks, Class<?> javaClass, Method method, String name) {
+        MethodInstaller invoker = (MethodInstaller) methodCallbacks.get(name);
+        if (invoker == null) {
+            invoker = new InstanceMethodInvokerInstaller(name);
+            methodCallbacks.put(name, invoker);
+        }
+        invoker.addMethod(method, javaClass);
+    }
+
+    private void installStaticMethods(Map<String, NamedInstaller> methodCallbacks, Class<?> javaClass, Method method, String name) {
+        MethodInstaller invoker = (MethodInstaller) methodCallbacks.get(name);
+        if (invoker == null) {
+            invoker = new StaticMethodInvokerInstaller(name);
+            methodCallbacks.put(name, invoker);
+        }
+        invoker.addMethod(method, javaClass);
+    }
     
     // old (quasi-deprecated) interface class
     private void setupInterfaceProxy(final RubyClass proxy) {
@@ -1695,20 +1712,29 @@ public class JavaClass extends JavaObject {
     }
     
     private static Method[] getMethods(Class<?> javaClass) {
-        ArrayList<Method> list = new ArrayList<Method>();
-        
+        HashMap<String, Method> nameMethods = new HashMap<String, Method>();
+
+        // we all all superclasses, but avoid adding superclass methods with same name+signature as subclass methods
+        // see JRUBY-3130
         for (Class c = javaClass; c != null; c = c.getSuperclass()) {
             try {
                 for (Method m : c.getDeclaredMethods()) {
                     int modifiers = m.getModifiers();
                     if (Modifier.isPublic(modifiers) || Modifier.isProtected(modifiers)) {
-                        list.add(m);
+                        String namePlusSig = m.getName() + CodegenUtils.sigParams(m.getParameterTypes());
+                        if (nameMethods.containsKey(namePlusSig)) continue;
+                        nameMethods.put(namePlusSig, m);
                     }
                 }
             } catch (SecurityException e) {
             }
         }
 
-        return list.toArray(new Method[list.size()]);
+        ArrayList<Method> list2 = new ArrayList<Method>();
+        for (Map.Entry<String, Method> entry : nameMethods.entrySet()) {
+            list2.add(entry.getValue());
+        }
+        
+        return list2.toArray(new Method[list2.size()]);
     }
 }
