@@ -39,23 +39,26 @@ import org.jruby.RubyFixnum;
 import org.jruby.RubyModule;
 import org.jruby.RubyObject;
 import org.jruby.RubyString;
+import org.jruby.anno.JRubyMethod;
+import org.jruby.anno.JRubyClass;
 import org.jruby.runtime.Block;
-import org.jruby.runtime.CallbackFactory;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ObjectMarshal;
+import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 
 /**
  *
  * @author  jpetersen
  */
+@JRubyClass(name="Java::JavaObject")
 public class JavaObject extends RubyObject {
+
     private static Object NULL_LOCK = new Object();
-    private final Object value;
 
     protected JavaObject(Ruby runtime, RubyClass rubyClass, Object value) {
         super(runtime, rubyClass);
-        this.value = value;
+        this.dataStruct = value;
     }
 
     protected JavaObject(Ruby runtime, Object value) {
@@ -63,141 +66,147 @@ public class JavaObject extends RubyObject {
     }
 
     public static JavaObject wrap(Ruby runtime, Object value) {
-        Object lock = value == null ? NULL_LOCK : value;
-        
-        synchronized (lock) {
-            JavaObject wrapper = runtime.getJavaSupport().getJavaObjectFromCache(value);
-            if (wrapper == null) {
-            	if (value == null) {
-            		wrapper = new JavaObject(runtime, value);
-            	} else if (value.getClass().isArray()) {
-                	wrapper = new JavaArray(runtime, value);
-                } else if (value.getClass().equals(Class.class)) {
-                	wrapper = JavaClass.get(runtime, (Class)value);
-                } else {
-                	wrapper = new JavaObject(runtime, value);
-                }
-                runtime.getJavaSupport().putJavaObjectIntoCache(wrapper);
+        if (value != null) {
+            if (value instanceof Class) {
+                return JavaClass.get(runtime, (Class<?>) value);
+            } else if (value.getClass().isArray()) {
+                return new JavaArray(runtime, value);
             }
-            return wrapper;
         }
+        return new JavaObject(runtime, value);
     }
 
-    public Class getJavaClass() {
-        return value != null ? value.getClass() : Void.TYPE;
+    public Class<?> getJavaClass() {
+        return dataStruct != null ? dataStruct.getClass() : Void.TYPE;
     }
 
     public Object getValue() {
-        return value;
+        return dataStruct;
     }
 
     public static RubyClass createJavaObjectClass(Ruby runtime, RubyModule javaModule) {
         // FIXME: Ideally JavaObject instances should be marshallable, which means that
         // the JavaObject metaclass should have an appropriate allocator. JRUBY-414
-    	RubyClass result = javaModule.defineClassUnder("JavaObject", runtime.getObject(), ObjectAllocator.NOT_ALLOCATABLE_ALLOCATOR);
+        RubyClass result = javaModule.defineClassUnder("JavaObject", runtime.getObject(), ObjectAllocator.NOT_ALLOCATABLE_ALLOCATOR);
 
-    	registerRubyMethods(runtime, result);
+        registerRubyMethods(runtime, result);
 
         result.getMetaClass().undefineMethod("new");
         result.getMetaClass().undefineMethod("allocate");
-        
+
         result.setMarshal(ObjectMarshal.NOT_MARSHALABLE_MARSHAL);
 
         return result;
     }
 
-	protected static void registerRubyMethods(Ruby runtime, RubyClass result) {
-		CallbackFactory callbackFactory = runtime.callbackFactory(JavaObject.class);
-
-        result.defineFastMethod("to_s", callbackFactory.getFastMethod("to_s"));
-        result.defineFastMethod("==", callbackFactory.getFastMethod("equal", IRubyObject.class));
-        result.defineFastMethod("eql?", callbackFactory.getFastMethod("equal", IRubyObject.class));
-        result.defineFastMethod("equal?", callbackFactory.getFastMethod("same", IRubyObject.class));
-        result.defineFastMethod("hash", callbackFactory.getFastMethod("hash"));
-        result.defineFastMethod("java_type", callbackFactory.getFastMethod("java_type"));
-        result.defineFastMethod("java_class", callbackFactory.getFastMethod("java_class"));
-        result.defineFastMethod("java_proxy?", callbackFactory.getFastMethod("is_java_proxy"));
-        result.defineMethod("synchronized", callbackFactory.getMethod("ruby_synchronized"));
-        result.defineFastMethod("length", callbackFactory.getFastMethod("length"));
-        result.defineFastMethod("[]", callbackFactory.getFastMethod("aref", IRubyObject.class));
-        result.defineFastMethod("[]=", callbackFactory.getFastMethod("aset", IRubyObject.class, IRubyObject.class));
-        result.defineFastMethod("fill", callbackFactory.getFastMethod("afill", IRubyObject.class, IRubyObject.class, IRubyObject.class));
-	}
-
-	public RubyFixnum hash() {
-        return getRuntime().newFixnum(value == null ? 0 : value.hashCode());
+    protected static void registerRubyMethods(Ruby runtime, RubyClass result) {
+        result.defineAnnotatedMethods(JavaObject.class);
     }
 
+    public boolean equals(Object other) {
+        return other instanceof JavaObject &&
+                this.dataStruct == ((JavaObject) other).dataStruct;
+    }
+
+    public int hashCode() {
+        if (dataStruct != null) {
+            return dataStruct.hashCode();
+        }
+        return 0;
+    }
+
+    @JRubyMethod
+    public RubyFixnum hash() {
+        return getRuntime().newFixnum(hashCode());
+    }
+
+    @JRubyMethod
     public IRubyObject to_s() {
-        String s = value == null ? "" : value.toString();
+        if (dataStruct != null) {
+            String stringValue = dataStruct.toString();
+            if (stringValue != null) {
+                return RubyString.newUnicodeString(getRuntime(), dataStruct.toString());
+            }
 
-        return s == null ? getRuntime().getNil() : RubyString.newUnicodeString(getRuntime(), s);
+            return getRuntime().getNil();
+        }
+        return RubyString.newEmptyString(getRuntime());
     }
 
-    public IRubyObject equal(IRubyObject other) {
+    @JRubyMethod(name = {"==", "eql?"}, required = 1)
+    public IRubyObject op_equal(IRubyObject other) {
         if (!(other instanceof JavaObject)) {
-            other = other.getInstanceVariable("@java_object");
+            other = (JavaObject)other.dataGetStruct();
             if (!(other instanceof JavaObject)) {
                 return getRuntime().getFalse();
             }
         }
-    	
+
         if (getValue() == null && ((JavaObject) other).getValue() == null) {
             return getRuntime().getTrue();
         }
-    	
+
         boolean isEqual = getValue().equals(((JavaObject) other).getValue());
         return isEqual ? getRuntime().getTrue() : getRuntime().getFalse();
     }
-    
+
+    @JRubyMethod(name = "equal?", required = 1)
     public IRubyObject same(IRubyObject other) {
         if (!(other instanceof JavaObject)) {
-            other = other.getInstanceVariable("@java_object");
+            other = (JavaObject)other.dataGetStruct();
             if (!(other instanceof JavaObject)) {
-              return getRuntime().getFalse();
+                return getRuntime().getFalse();
             }
         }
-      
+
         if (getValue() == null && ((JavaObject) other).getValue() == null) {
             return getRuntime().getTrue();
         }
-      
+
         boolean isSame = getValue() == ((JavaObject) other).getValue();
         return isSame ? getRuntime().getTrue() : getRuntime().getFalse();
     }
 
+    @JRubyMethod
     public RubyString java_type() {
         return getRuntime().newString(getJavaClass().getName());
     }
 
+    @JRubyMethod
     public IRubyObject java_class() {
         return JavaClass.get(getRuntime(), getJavaClass());
     }
 
+    @JRubyMethod
     public RubyFixnum length() {
         throw getRuntime().newTypeError("not a java array");
     }
 
+    @JRubyMethod(name = "[]", required = 1)
     public IRubyObject aref(IRubyObject index) {
         throw getRuntime().newTypeError("not a java array");
     }
 
+    @JRubyMethod(name = "[]=", required = 2)
     public IRubyObject aset(IRubyObject index, IRubyObject someValue) {
         throw getRuntime().newTypeError("not a java array");
     }
-    
-    public IRubyObject afill (IRubyObject beginIndex, IRubyObject endIndex, IRubyObject someValue) {
+
+    @JRubyMethod(name = "fill", required = 3)
+    public IRubyObject afill(IRubyObject beginIndex, IRubyObject endIndex, IRubyObject someValue) {
         throw getRuntime().newTypeError("not a java array");
     }
-    
+
+    @JRubyMethod(name = "java_proxy?")
     public IRubyObject is_java_proxy() {
         return getRuntime().getTrue();
     }
 
-    public IRubyObject ruby_synchronized(Block block) {
+    @JRubyMethod(name = "synchronized")
+    public IRubyObject ruby_synchronized(ThreadContext context, Block block) {
         Object lock = getValue();
         synchronized (lock != null ? lock : NULL_LOCK) {
-            return block.yield(getRuntime().getCurrentContext(), null);
+            return block.yield(context, null);
         }
     }
 }
