@@ -62,10 +62,11 @@ import org.jruby.RubyTime;
 import org.jruby.internal.runtime.methods.CallConfiguration;
 import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.java.proxies.JavaProxy;
+import org.jruby.javasupport.proxy.InternalJavaProxy;
+import org.jruby.javasupport.proxy.JavaProxyInvocationHandler;
 import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ClassIndex;
-import org.jruby.runtime.MethodIndex;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -527,7 +528,11 @@ public class JavaUtil {
              	return new BigDecimal(double_value);
          	}
         }
+
         try {
+            if (isDuckTypeConvertable(rubyObject.getClass(), javaClass)) {
+                return convertProcToInterface(context, (RubyObject) rubyObject, javaClass);
+            }
             return ((JavaObject) rubyObject).getValue();
         } catch (ClassCastException ex) {
             if (rubyObject.getRuntime().getDebug().isTrue()) ex.printStackTrace();
@@ -741,6 +746,15 @@ public class JavaUtil {
             return (IRubyObject)object;
         }
         
+        if (object instanceof InternalJavaProxy) {
+            InternalJavaProxy internalJavaProxy = (InternalJavaProxy) object;
+            IRubyObject orig = internalJavaProxy.___getInvocationHandler().getOrig();
+
+            if (orig != null) {
+                return orig;
+            }
+        }
+        
         JavaConverter converter = JAVA_CONVERTERS.get(object.getClass());
         if (converter == null || converter == JAVA_DEFAULT_CONVERTER) {
             return Java.getInstance(runtime, object);
