@@ -36,6 +36,7 @@ import java.lang.reflect.Modifier;
 import java.lang.reflect.UndeclaredThrowableException;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
+import java.security.ProtectionDomain;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
@@ -44,6 +45,7 @@ import java.util.Iterator;
 import java.util.Map;
 import java.util.Set;
 
+import org.jruby.Ruby;
 import org.objectweb.asm.ClassVisitor;
 import org.objectweb.asm.ClassWriter;
 import org.objectweb.asm.FieldVisitor;
@@ -106,11 +108,18 @@ public class JavaProxyClassFactory {
         return counter++;
     }
     
+    @Deprecated
+    static JavaProxyClass newProxyClass(ClassLoader loader,
+            String targetClassName, Class superClass, Class[] interfaces, Set names)
+            throws InvocationTargetException {
+        return newProxyClass(JavaProxyClass.runtimeTLS.get(), loader, targetClassName, superClass, interfaces, names);
+    }
+    
     // TODO: we should be able to optimize this quite a bit post-1.0.  JavaClass already
     // has all the methods organized by method name; the next version (supporting protected
     // methods/fields) will have them organized even further. So collectMethods here can
     // just lookup the overridden methods in the JavaClass map, should be much faster.
-    static JavaProxyClass newProxyClass(ClassLoader loader,
+    static JavaProxyClass newProxyClass(Ruby runtime, ClassLoader loader,
             String targetClassName, Class superClass, Class[] interfaces, Set names)
             throws InvocationTargetException {
         if (loader == null) {
@@ -141,25 +150,21 @@ public class JavaProxyClassFactory {
         if (proxyClass == null) {
 
             if (targetClassName == null) {
-                String pkg = packageName(superClass);
+                // We always prepend an org.jruby.proxy package to the beginning
+                // because java and javax packages are protected and signed
+                // jars prevent us generating new classes with those package
+                // names. See JRUBY-2439.
+                String pkg = proxyPackageName(superClass);
                 String fullName = superClass.getName();
                 int ix = fullName.lastIndexOf('.');
                 String cName = fullName;
                 if(ix != -1) {
                     cName = fullName.substring(ix+1);
                 }
-                if (pkg.startsWith("java.") || pkg.startsWith("javax.")) {
-                    pkg = packageName(JavaProxyClassFactory.class) + ".gen";
-                }
-                if(ix == -1) {
-                    targetClassName = cName + "$Proxy" + nextId();
-                } else {
-                    targetClassName = pkg + "." + cName + "$Proxy"
-                        + nextId();
-                }
+                targetClassName = pkg + "." + cName + "$Proxy" + nextId();
             }
 
-            validateArgs(targetClassName, superClass);
+            validateArgs(runtime, targetClassName, superClass);
 
             Map methods = new HashMap();
             collectMethods(superClass, interfaces, methods, names);
@@ -233,7 +238,7 @@ public class JavaProxyClassFactory {
                 try {
                     defineClass_method = ClassLoader.class.getDeclaredMethod(
                             "defineClass", new Class[] { String.class,
-                                    byte[].class, int.class, int.class });
+                                    byte[].class, int.class, int.class, ProtectionDomain.class });
                 } catch (Exception e) {
                     // should not happen!
                     e.printStackTrace();
@@ -250,7 +255,7 @@ public class JavaProxyClassFactory {
         try {
             return (Class) defineClass_method
                     .invoke(loader, new Object[] { className, data,
-                            new Integer(0), new Integer(data.length) });
+                            new Integer(0), new Integer(data.length), JavaProxyClassFactory.class.getProtectionDomain() });
         } catch (IllegalArgumentException e) {
             // TODO Auto-generated catch block
             e.printStackTrace();
@@ -269,7 +274,7 @@ public class JavaProxyClassFactory {
     private static ClassWriter beginProxyClass(final String targetClassName,
             final Class superClass, final Class[] interfaces) {
 
-        ClassWriter cw = new ClassWriter(true);
+        ClassWriter cw = new ClassWriter(ClassWriter.COMPUTE_MAXS);
 
         int access = Opcodes.ACC_PUBLIC | Opcodes.ACC_FINAL | Opcodes.ACC_STATIC;
         String name = toInternalClassName(targetClassName);
@@ -333,7 +338,7 @@ public class JavaProxyClassFactory {
 
     private static void generateConstructors(Class superClass, Type selfType,
             ClassVisitor cw) {
-        Constructor[] cons = superClass.getConstructors();
+        Constructor[] cons = superClass.getDeclaredConstructors();
         for (int i = 0; i < cons.length; i++) {
             Constructor constructor = cons[i];
 
@@ -569,7 +574,7 @@ public class JavaProxyClassFactory {
         }
 
         public String scrambledSignature() {
-            StringBuffer sb = new StringBuffer();
+            StringBuilder sb = new StringBuilder();
             Class[] parms = getParameterTypes();
             for (int i = 0; i < parms.length; i++) {
                 sb.append('$');
@@ -765,25 +770,23 @@ public class JavaProxyClassFactory {
         }
     }
 
-    private static void validateArgs(String targetClassName, Class superClass) {
+    private static void validateArgs(Ruby runtime, String targetClassName, Class superClass) {
 
         if (Modifier.isFinal(superClass.getModifiers())) {
-            throw new IllegalArgumentException("cannot extend final class");
+            throw runtime.newTypeError("cannot extend final class " + superClass.getName());
         }
 
         String targetPackage = packageName(targetClassName);
 
         String pkg = targetPackage.replace('.', '/');
         if (pkg.startsWith("java")) {
-            throw new IllegalArgumentException("cannor add classes to package "
-                    + pkg);
+            throw runtime.newTypeError("cannot add classes to package " + pkg);
         }
 
         Package p = Package.getPackage(pkg);
         if (p != null) {
             if (p.isSealed()) {
-                throw new IllegalArgumentException("package " + p
-                        + " is sealed");
+                throw runtime.newTypeError("package " + p + " is sealed");
             }
         }
     }
@@ -802,4 +805,14 @@ public class JavaProxyClassFactory {
         }
     }
 
+    private static String proxyPackageName(Class clazz) {
+        String clazzName = clazz.getName();
+        int idx = clazzName.lastIndexOf('.');
+        if (idx == -1) {
+            return "org.jruby.proxy";
+        } else {
+            return "org.jruby.proxy." + clazzName.substring(0, idx);
+        }
+    }
+
 }
