@@ -15,7 +15,7 @@
  * Copyright (C) 2001-2002 Jan Arne Petersen <jpetersen@uni-bonn.de>
  * Copyright (C) 2002 Anders Bengtsson <ndrsbngtssn@yahoo.se>
  * Copyright (C) 2004 Charles O Nutter <headius@headius.com>
- * 
+ *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
  * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
@@ -31,6 +31,7 @@
 package org.jruby.runtime;
 
 import java.io.IOException;
+import java.io.InputStream;
 import java.util.Properties;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
@@ -41,24 +42,48 @@ public final class Constants {
 
     public static final int MARSHAL_MAJOR = 4;
     public static final int MARSHAL_MINOR = 8;
-    
+
     public static final String RUBY_MAJOR_VERSION;
     public static final String RUBY_VERSION;
+    public static final String RUBY1_9_MAJOR_VERSION;
+    public static final String RUBY1_9_VERSION;
+    public static final String RUBY_PATCHLEVEL;
+
     public static final String COMPILE_DATE;
     public static final String VERSION;
     public static final String BUILD;
     public static final String TARGET;
     public static final String REVISION;
+    public static final String ENGINE = "jruby";
+
+    public static final String JRUBY_PROPERTIES = "/org/jruby/jruby.properties";
 
     static {
+        InputStream stream = null;
         try {
-            properties.load(Constants.class.getResourceAsStream("/jruby.properties"));
+            stream = Constants.class.getResourceAsStream(JRUBY_PROPERTIES);
+            if (stream == null) {
+                throw new RuntimeException("Resource not found: " + JRUBY_PROPERTIES);
+            }
+            properties.load(stream);
         } catch (IOException ioe) {
             ioe.printStackTrace();
+        } finally {
+            if (stream != null) {
+                try {
+                    stream.close();
+                } catch (IOException e) {
+                    // silently ignore
+                }
+            }
         }
-        
+
         RUBY_MAJOR_VERSION = properties.getProperty("version.ruby.major");
         RUBY_VERSION = properties.getProperty("version.ruby");
+        RUBY_PATCHLEVEL = properties.getProperty("version.ruby.patchlevel");
+
+        RUBY1_9_MAJOR_VERSION = properties.getProperty("version.ruby1_9.major");
+        RUBY1_9_VERSION = properties.getProperty("version.ruby1_9");
         COMPILE_DATE = properties.getProperty("release.date");
         VERSION = properties.getProperty("version.jruby");
         BUILD = properties.getProperty("build.jruby");
@@ -67,9 +92,9 @@ public final class Constants {
         if (matcher.find()) {
             REVISION = matcher.group(1);
         } else {
-            REVISION = "unknown";
+            REVISION = "0000";
         }
     }
-    
+
     private Constants() {}
 }
