@@ -337,6 +337,18 @@ test_equal("a message", e3.message)
 test_equal(e1.backtrace, e3.backtrace)
 test_equal(42, e3.ivar)
 
+# Check that numbers of all sizes don't make link indexes break
+fixnum = 12345
+mri_bignum = 1234567890
+jruby_bignum = 12345678901234567890
+s = "should be cached"
+a = [fixnum,mri_bignum,jruby_bignum,s]
+a = a + a
+dumped_by_mri = "\004\b[\ri\00290l+\a\322\002\226Il+\t\322\n\037\353\214\251T\253\"\025should be cachedi\00290l+\a\322\002\226I@\a@\b"
+test_no_exception { test_equal(a, Marshal.load(Marshal.dump(a))) }
+test_no_exception { test_equal(a, Marshal.load(dumped_by_mri)) }
+test_no_exception { test_equal(dumped_by_mri, Marshal.dump(a)) }
+
 require 'stringio'
 class MTStream < StringIO
   attr :binmode_called
@@ -390,4 +402,83 @@ Marshal.load(in_stream)
 test_ok(in_stream.binmode_called)
 
 # thread isn't marshalable
-test_exception(TypeError) { Marshal.dump(Thread.new {}) }
\ No newline at end of file
+test_exception(TypeError) { Marshal.dump(Thread.new {}) }
+
+# time marshalling
+unpacked_marshaled_time = [4, 8, 117, 58, 9, 84, 105, 109, 101, 13, 247, 239, 26, 128, 57, 48, 112, 57]
+actual_time = Time.utc(2007, 12, 31, 23, 14, 23, 12345)
+test_equal(
+  unpacked_marshaled_time,
+  Marshal.dump(actual_time).unpack('C*'))
+test_equal(
+  0,
+  actual_time <=> Marshal.load(unpacked_marshaled_time.pack('C*')))
+
+# JRUBY-2392
+time = Time.now
+is_utc = time.to_s =~ /UTC/
+if (is_utc)
+  test_ok(Marshal.load(Marshal.dump(time)).to_s =~ /UTC/)
+else
+  test_ok(!(Marshal.load(Marshal.dump(time)).to_s =~ /UTC/))
+end
+
+# JRUBY-2345
+begin
+  file_name = "test-file-tmp"
+  f = File.new(file_name, "w+")
+  test_exception(EOFError) { Marshal.load(f) }
+ensure
+  f.close
+  File.delete(file_name)
+end
+
+test_exception(ArgumentError) { Marshal.load("\004\b\"\b") }
+
+# borrowed from MRI 1.9 test:
+class C
+  def initialize(str)
+    @str = str
+  end
+  def _dump(limit)
+    @str
+  end
+  def self._load(s)
+    new(s)
+  end
+end
+test_exception(ArgumentError) {
+  (data = Marshal.dump(C.new("a")))[-2, 1] = "\003\377\377\377"
+  Marshal.load(data)
+}
+
+# JRUBY-2975: Overriding Time._dump does not behave the same as MRI
+class Time
+  class << self
+    alias_method :_original_load, :_load
+    def _load(marshaled_time)
+      time = _original_load(marshaled_time)
+      utc = time.send(:remove_instance_variable, '@marshal_with_utc_coercion')
+      utc ? time.utc : time
+    end
+  end
+
+  alias_method :_original_dump, :_dump
+  def _dump(*args)
+    obj = self.frozen? ? self.dup : self
+    obj.instance_variable_set('@marshal_with_utc_coercion', utc?)
+    obj._original_dump(*args)
+  end
+end
+
+t = Time.local(2000).freeze
+t2 = Marshal.load(Marshal.dump(t))
+test_equal t, t2
+
+# reset _load and _dump
+class Time
+  class << self
+    alias_method :load, :_original_load
+  end
+  alias_method :dump, :_original_dump
+end
\ No newline at end of file
