@@ -9,7 +9,7 @@ def short_name(long_name)
 end
 
 def node_string(node)
-  return "#{short_name(node.java_class.name)}" unless node.position
+  return "#{short_name(node.java_class.name)}" if node.invisible?
   p = node.position
   "#{short_name(node.java_class.name)},#{p.startLine},#{p.endLine},#{p.startOffset},#{p.endOffset}"
 end
@@ -18,24 +18,32 @@ def test_pos_ok(a, b, entry, node, test_name)
   test_ok(a == b, "Expected [#{entry.join(',')}] but got [#{node_string(node)}] for #{test_name}")
 end
 
+$table = {}
+AST = org.jruby.ast
+
 def compare_node(node, list, test_name)
+  unless node.invisible?
     entry = list.delete_at(0)
     
     unless entry.nil? 
+      cname = entry[0]
+      
+      $table[cname] = eval "AST.#{cname}" unless $table[cname]
       position = node.position
     
-      test_pos_ok(entry[0], short_name(node.java_class.name), entry, node, test_name)
+      test_ok(node.class.ancestors.include?($table[cname]), "Wrong Class (Expected: #{cname}. Got: #{short_name(node.java_class.name)} in [#{test_name}]")
       test_pos_ok(entry[1], position.startLine, entry, node, test_name)
       test_pos_ok(entry[2], position.endLine, entry, node, test_name)
       test_pos_ok(entry[3], position.startOffset, entry, node, test_name)
       test_pos_ok(entry[4], position.endOffset, entry, node, test_name)
     end
+  end
 	
-    node.childNodes.each {|child| compare_node(child, list, test_name) }
+  node.childNodes.each {|child| compare_node(child, list, test_name) }
 end
 
 def test_tree(expected_list, script_content, test_name=nil, verbose=false)
-   root = JRuby::parse(script_content, "")
+   root = JRuby::parse(script_content, "", true)
    print_tree(root) if verbose
    compare_node(root, expected_list, test_name)
 end
@@ -781,7 +789,7 @@ nil, #['NewlineNode',0,3,0,22],
     ['EvStrNode',1,1,10,17],
       nil, #['NewlineNode',1,1,12,18],
         ['VCallNode',1,1,12,16],
-    ['StrNode',1,2,17,21]
+    ['StrNode',1,2,17,18]
 ]
 
 test_tree(list, <<'END', "heredoc with nested expression")
@@ -1022,6 +1030,7 @@ nil, #['NewlineNode',0,1,0,9],
   ['DefnNode',0,0,0,25],
     ['ArgumentNode',0,0,4,5],
     ['ArgsNode',0,0,5,11],
+    ['ArgumentNode',0,0,6,10],
       nil, #['NewlineNode',0,0,12,25],
         ['FCallNode',0,0,12,21],
           ['SplatNode',0,0,15,21],
@@ -1505,6 +1514,7 @@ nil, #['NewlineNode', 0, 2, 0, 35],
 ['ArgumentNode', 0, 0, 9, 10],
 ['ArgumentNode', 0, 0, 12, 13],
 ['ArgumentNode', 0, 0, 15, 16],
+['ArgumentNode', 0, 0, 18, 22],
 ['BlockArgNode', 0, 0, 24, 30],
 nil, #['ScopeNode', 1, 1, 33, 34]
 ]
@@ -1519,7 +1529,7 @@ nil,
 ['LocalAsgnNode', 0, 0, 0, 15],
 ['CallNode', 0, 0, 4, 15],
 ['FixnumNode', 0, 0, 4, 5],
-['ArrayNode', 0, 0, 9, 15],
+['ArrayNode', 0, 0, 8, 15],
 nil,
 ['CallNode', 0, 0, 9, 14],
 ['FixnumNode', 0, 0, 9, 10],
@@ -1691,3 +1701,48 @@ test_tree(list, <<END)
 def []
 end
 END
+
+list = [
+nil,
+  nil,
+    ['CallNode',0,0,0,9],
+      nil,
+        ['VCallNode',0,0,1,2],
+      ['ArrayNode',0,0,7,9],
+        ['FixnumNode',0,0,7,9]
+]
+test_tree(list, <<END, "call in parens")
+(x) >> 10
+END
+
+list = [
+nil,
+  nil,
+    ['YieldNode',0,0,0,7],
+      ['VCallNode',0,0,6,7]
+]
+test_tree(list, <<END, "bare yield")
+yield x
+END
+
+def heh; end
+
+list = [
+['RootNode',0,3,0,32],
+  ['NewlineNode',0,3,0,32],
+    ['LocalAsgnNode',0,3,0,32],
+      ['SValueNode',0,3,1,32],
+        ['ArrayNode',0,2,4,31],
+          ['StrNode',0,2,4,21],
+            ['StrNode',2,2,23,31],
+            ['EvStrNode',1,1,23,31],
+              ['NewlineNode',1,1,25,29],
+                ['VCallNode',1,1,25,28],
+            ['StrNode',1,2,29,34],
+          ['StrNode',2,2,29,37]
+]
+test_tree(list, <<END, "heredoc positions")
+a = <<-STR, __FILE__
+  #{heh} boo
+STR
+END
