@@ -36,6 +36,12 @@ test_exception(ArgumentError) { s.new(1,2,3,4,5,6,7) }
 test_exception(ArgumentError) { Struct.new }
 test_exception(NameError) { Struct.new('foo', 'bar') }
 
+# name should be coerced to string with to_str
+foo = Object.new
+def foo.to_str() 'Foo' end
+s = Struct.new(foo, :x, :y)
+test_equal s, Struct::Foo
+
 # Anonymous Struct
 a = Struct.new(:x, :y)
 a1 = a.new(5, 7)
@@ -112,4 +118,28 @@ s2 = c.new(1)
 test_ok(true, s1.eql?(s2))
 test_equal(s1.hash, s2.hash)
 
-test_no_exception { Struct.new(:icandup).new(1).dup }
\ No newline at end of file
+test_no_exception { Struct.new(:icandup).new(1).dup }
+
+FiveElementStruct = Struct.new(:a, :b, :c, :d, :e)
+fes = FiveElementStruct.new(1, 2, 3, 4, 5)
+test_equal([2,4], fes.select {|i| (i % 2).zero?})
+
+# JRUBY-2157
+class Foo < Struct.new(:heh)
+  def initialize
+  end
+end
+
+test_equal(nil, Foo.new.heh)
+
+
+# JRUBY-2490
+require 'java'
+
+class JavaComparableStruct < Struct.new(:foo)
+  include java.lang.Comparable
+  
+  def compare_to(other); 0; end
+end
+
+test_equal(:a, JavaComparableStruct.new(:a).foo)
\ No newline at end of file
