@@ -49,7 +49,7 @@ test_equal(nil, t == nil)
 
 # Time.utc can accept float values (by turning them into ints)
 test_no_exception { Time::utc(1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0) }
-test_exception(ArgumentError) { Time::local(1111, 11, 11, 11, 11, 11, 0) }
+test_exception(ArgumentError) { Time::local(2**30, 11, 11, 11, 11, 11, 0) }
 
 # Test getgm/local/utc methods
 local = Time.now
@@ -59,6 +59,12 @@ utc = local2.getutc
 test_equal(local, local2)
 test_equal(gmt, utc)
 
+# time with usecs (JRUBY-1971)
+local = Time.local(2008, 12, 30, 5, 30, 0, 100)
+gmt = local.getgm
+local2 = gmt.getlocal
+test_equal(local, local2)
+
 test_exception { Time::utc(nil,nil,nil,nil,nil,nil,0) }
 
 # Sat Jan  1 14:58:42 2000
@@ -105,3 +111,28 @@ test_equal "12:00PM", Time.utc(2007,01,01,12,0).strftime("%I:%M%p")
 # MRI only accepts usec arguments that fit in signed int
 test_exception {Time.utc(2001,2,3,4,5,6,0x80000000)}
 test_exception {Time.utc(2001,2,3,4,5,6,-0x80000001)}
+
+# Time.utc accepts 8 arguments for compatibility with parsedate (two last arguments are ignored)
+test_no_exception {Time.utc(2007, 10, 10, 11, 55, 23, "-0200", 3)}
+
+# Time.utc accepts negative usec and rolls over
+test_equal 999999,  Time.utc(2007, 10, 10, 11, 55, 23, -1).usec
+
+# Follow MRI year conversion logic
+test_equal 2000, Time.utc(0).year
+test_equal 1969, Time.utc(69).year
+test_equal 2038, Time.utc(38).year
+test_equal 2038, Time.utc(138).year
+test_equal 1902, Time.utc(1902).year
+
+=begin Disabled, see http://jira.codehaus.org/browse/JRUBY-3079
+old_tz = ENV['TZ']
+begin
+  ENV['TZ']='Europe/Helsinki'
+  test_equal "+0300 EEST", Time.now.strftime('%z %Z')
+  ENV['TZ']='America/New_York'
+  test_equal "-0400 EDT", Time.now.strftime('%z %Z')
+ensure
+  ENV['TZ'] = old_tz
+end
+=end
\ No newline at end of file
