@@ -258,4 +258,30 @@ class TestThread < Test::Unit::TestCase
       t.wakeup; t.join
     end
   end
+
+  # JRUBY-2380: Thread.list has a race condition
+  # Fix is to make sure the thread is added to the global list before returning from Thread#new
+  def test_new_thread_in_list
+    count = 10
+    live = Thread.list.size
+
+    100.times do
+      threads = []
+      count.times do
+        threads << Thread.new do
+          sleep
+        end
+      end
+
+      if (size = Thread.list.size) != count + live
+        raise "wrong! (expected #{count + live} but was #{size})"
+      end
+
+      threads.each do |t|
+        Thread.pass until t.status == 'sleep'
+        t.wakeup
+        t.join
+      end
+    end
+  end
 end
